
* Note: fixed a bug in nl.ado where after 
*    di as error "nl`expr' returned " _rc 
* I added 
*    exit_rc
* where it was missing (it then didn't interrupt when hitting Ctrl-break)
* will have to redo with new version of stata (done for version 10)
* (but bug may be fixed in newer versions of Stata) 


version 10.1
drop _all
set memory 512m
set more off
set matsize 800
set type double, permanently

//set input and output directories
local dr "/Users/stenagel"
local outdir "`dr'/Dropbox/temp/Inflexp/"
cd "`dr'/Dropbox/temp/Inflexp"

global qtr = 1   // 0 for annual inflation data in inputs, 1 for quarterly (annualized) data




// --------------------------------------------------------------------------- //
//																			   //	
// Prepare data 															   //	
//																			   //	
// --------------------------------------------------------------------------- //


// load macro data into matrix 

insheet using "`outdir'infl.txt", clear

gen yyyy = floor(yyyyqtr/100)
gen q = yyyyqtr-yyyy*100

mkmat yyyyqtr
mkmat yrinfl
mkmat yrret
mkmat inflelderly
mkmat inflgas     // has missing values in 1960s, this is correct, missing in downloaded series
mkmat spf
mkmat spf10


local m = rowsof(yrinfl)
local rowyrs ""

forvalues i=1/`m'  {
   local addyr = yyyyqtr[`i',1]
   local rowyrs "`rowyrs' `addyr' "
   }

matrix rownames yrinfl = `rowyrs'
matrix rownames yrret = `rowyrs'
matrix rownames inflelderly = `rowyrs'
matrix rownames inflgas = `rowyrs'
matrix rownames spf = `rowyrs'
matrix rownames spf10 = `rowyrs'

save macrofile, replace






// --------------------------------------------------------------------------- //
//																			   //	
// Define programs that are used in code below 								   //	
//																			   //	
// --------------------------------------------------------------------------- //



/******* programs for recursive updating ***************/


capture program drop constgain
program constgain
   version 10.1
   syntax anything
   tempname phit pf xf gcoh infl linfl llinfl ao lao dinfl dlinfl ageqtr ///
            c cc x xcurr y R smean smeanx perr1 perr2 perr3 perr4 g infl4 linfl4 l4infl ac winfl4 wlinfl4
   scalar `g' = `anything'
   matrix inflfullexp = J((2009-1871)*4,1,-999) 
   matrix acfull = J((2009-1871)*4,1,-999)
   matrix mafull = J((2009-1871)*4,1,-999)
   
   local yrst = $yrst
   forvalues yr = `yrst'/2009 {
      forvalues q = 1/4 {
          local yri = (`yr'-1872)*4+`q'      // yrinfl starts in qtr. 1872:01
          scalar `infl' = $pred[`yri',1]
          
          //initialize learning process
          
             if (`yr' < `yrst'+20) {     //& `q' == 1
             
                if $gain < 4 {                       // simple weighted average                       
                   matrix `R' = 1
                   //matrix `x1' = 1   
                   matrix `phit' = `infl'
                   matrix `cc' = 0
                   matrix `x' = 1
                   //matrix `x3' = 1
                   //matrix `x2' = 1 
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                }
                
                else if $gain == 4 {                          // Recursive least squares: AR(1)
                   //matrix `R' = (1, `infl'\ `infl', `infl'^2)  // dispersion in fitted exp looks identical with this one
                   matrix `R' = (1, 0\ 0, 0)
                   //matrix `x1' = (1,`infl')   
                   matrix `phit' = `infl'
                   matrix `cc' = (`phit' \ 0)     // dispersion in fitted exp looks identical with 0, 0.99 as starting value
                   matrix `x' = (1,0)
                   //matrix `x3' = (1,0)
                   //matrix `x2' = (1,0)      
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                }
                
                else if $gain == 5 {                          // Stochastic gradient: AR(1)
                   matrix `R' = 1
                   //matrix `x1' = (1,`infl')   
                   matrix `phit' = `infl'
                   matrix `cc' = (`phit' \ 0)
                   matrix `x' = (1,0)
                   //matrix `x3' = (1,0)
                   //matrix `x2' = (1,0) 
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                }
                
                else if $gain == 6 {                          // IMA(1,1) 
                   matrix `R' = (1, 0\ 0, 1)   // do not impose zero intercept in est., only in forecasting
                   matrix `phit' = `infl'
                   matrix `cc' =  (0 \ 0)
                   matrix `x' = (0, 0)
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                else if $gain == 7 {                          // Recursive least squares: ARMA(1,1) 
                   matrix `R' = (1, 0, 0 \ 0, 1, 0 \ 0, 0, 1)
                   matrix `phit' = `infl'
                   matrix `cc' = (0 \ 1 \ 0)     
                   matrix `x' = (1,0, 0)
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                else if $gain == 8 {                          // AR(2)
                   matrix `R' = (1, 0, 0 \ 0, 1, 0 \ 0, 0, 1)
                   matrix `phit' = `infl'
                   matrix `cc' = (0 \ 1 \ 0)     
                   matrix `x' = (1, 0, 0)
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                else if $gain == 9 {                          // MA(1)
                   matrix `R' = (1, 0\ 0, 0)
                   matrix `phit' = `infl'
                   matrix `cc' = (`phit' \ 0)     // dispersion in fitted exp looks identical with 0, 0.99 as starting value
                   matrix `x' = (1,0)
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                else if $gain == 10 { 
                   matrix `R' = 1    // (1, 0 \ 0, 1)
                   matrix `phit' = `infl'
                   matrix `cc' = 0     
                   matrix `x' = 0
                   scalar `smean' = 0
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                matrix `c' = `cc'
                
                   
             }
             
             
             //updating of parameter estimates 
             
             else  {
             
                if $qtr == 0 {
                   scalar `linfl' = $pred[`yri'-4,1]
                   scalar `llinfl' = $pred[`yri'-8,1]
                   scalar `dlinfl' = `linfl'-$pred[`yri'-8,1]
                   scalar `dinfl' = `infl'-$pred[`yri'-4,1]
                }
                else if $qtr == 1 {
                   scalar `linfl' = $pred[`yri'-1,1]
                   scalar `llinfl' = $pred[`yri'-2,1]
                   scalar `l4infl' = $pred[`yri'-4,1]
                   scalar `dlinfl' = `linfl'-$pred[`yri'-2,1]
                   scalar `dinfl' = `infl'-$pred[`yri'-1,1]
                   scalar `lao' = ($pred[`yri'-1,1]+$pred[`yri'-2,1]+$pred[`yri'-3,1]+$pred[`yri'-4,1])/4
                   scalar `ao' = ($pred[`yri',1]+$pred[`yri'-1,1]+$pred[`yri'-2,1]+$pred[`yri'-3,1])/4
                   scalar `wlinfl4' = (4*$pred[`yri'-1,1]+3*$pred[`yri'-2,1]+2*$pred[`yri'-3,1]+$pred[`yri'-4,1])/10
                   scalar `winfl4' = (4*$pred[`yri',1]+3*$pred[`yri'-1,1]+2*$pred[`yri'-2,1]+$pred[`yri'-3,1])/10
                   scalar `infl4' = ($pred[`yri',1]+$pred[`yri'-1,1]+$pred[`yri'-2,1]+$pred[`yri'-3,1])/4
                   scalar `linfl4' = ($pred[`yri'-4,1]+$pred[`yri'-5,1]+$pred[`yri'-6,1]+$pred[`yri'-7,1])/4
                }
                
                if $gain == 3 {
                   matrix `x' = 1
                }
                else if ($gain == 4 | $gain == 5) {
                   matrix `x' = (1,`linfl') 
                }
                else if $gain == 6 {
                   if $qtr == 0 { 
                      matrix `x' = (1,`perr4')
                   }   
                   else {
                      matrix `x' = (1,`perr1')
                   }   
                }
                else if $gain == 7 { 
                   if $qtr == 0 { 
                      matrix `x' = (1, `linfl', `perr4')
                   }   
                   else {
                      matrix `x' = (1, `linfl', `perr1')
                   }   
                }
                else if $gain == 8 { 
                   if $qtr == 0 { 
                      matrix `x' = (1, `linfl', `llinfl')   
                   }   
                   else {
                      matrix `x' = (1, `linfl', `llinfl')
                   }
                }
                else if $gain == 9 { 
                   if $qtr == 0 { 
                      matrix `x' = (1, `perr4')
                   }   
                   else {
                      matrix `x' = (1, `perr1')
                   }   
                }
                else if $gain == 10 { 
                   if $qtr == 0 { 
                      matrix `x' = (1, `lao')
                   }   
                   else {
                      matrix `x' = ( `wlinfl4')
                   }
                }
                
                if $gain == 10 {
                   scalar `y' = `infl'
                }
                else if $gain ~= 6 {
                   scalar `y' = `infl'
                }
                else {
                   scalar `y' = `dinfl'
                }
                
                if $gain > 5 {
                   matrix `perr4' = `perr3'
                   matrix `perr3' = `perr2'
                   matrix `perr2' = `perr1'
                }
                
                //if ($gain ~= 6 ) {
                   matrix `perr1' = `y'-`cc''*`x''
                //}   
                //else {
                //   matrix `perr1' = `y'-`cc'[2,1]*`x'[1,2] 
                //}
                //else { 
                //   matrix `perr1' = `infl'-`cc''*(1 \ `linfl' \ `perr1')   // this presumes RW interpretation
                   // matrix `perr1' = `dinfl'-`cc'[2,1]*`x'[1,3] 
                //}  
                   
                
                //recursive updating step
                //if $gain ~= 10 {
                  if $gain ~= 5 {
                     matrix `R' = `R' + `g'*(`x''*`x'-`R')      
                  }
                  matrix `cc' = `cc' + `g'*invsym(`R')*`x''*`perr1'     
                //}
                //else {
                //   matrix `R' = `R' + `gcoh'*(`x''*`x'-`R')     
                //   matrix `cc' = `cc' + `gcoh'*invsym(`R')*`x''*(`y'-`cc''*`x'')
                //}
                
                //parameter constraints
                matrix `c' = `cc'
                if ($gain == 6) {
                   matrix `c'[2,1] = max(`c'[2,1],-0.999)
                   matrix `c'[2,1] = min(`c'[2,1],0)
                }
                if $gain == 7 {
                   matrix `c'[2,1] = `c'[2,1]-max(`cc'[2,1]-1,0)   // important to do manipulations on `c', not `cc'
                   matrix `c'[2,1] = `c'[2,1]-min(`cc'[2,1],0)
                   matrix `c'[1,1] = `c'[1,1] + max(`cc'[2,1]-1,0)*`smeanx' + min(`cc'[2,1],0)*`smeanx'
                   matrix `c'[3,1] = max(`c'[3,1],-0.999)
                   matrix `c'[3,1] = min(`c'[3,1],0.999)
                }
                          
                scalar `smean' = `smean' + `g'*(`infl'-`smean') 
                scalar `smeanx' = `smeanx' + `g'*(`linfl'-`smeanx')  
                    
                if $gain == 3 {
                   matrix `xcurr' = 1 
                }
                else if ($gain == 4 | $gain == 5) {
                   matrix `xcurr' = (1,`infl') 
                }
                else if $gain == 6 {
                  matrix `xcurr' = (1,`perr1')
                }
                else if $gain == 7 {
                  matrix `xcurr' = (1, `infl', `perr1')
                }
                else if $gain == 8 {
                  matrix `xcurr' = (1, `infl', `linfl')
                }
                else if $gain == 9 {
                  matrix `xcurr' = (1, `perr1')
                }
                else if $gain == 10 {
                  matrix `xcurr' = ( `winfl4')
                }
                
                if ($qtr == 1 & $gain ~= 10)  {
                   local hf = 4*$hor-1
                }
                else {
                   local hf = $hor-1
                }
                  
                matrix `phit' = `c''*`xcurr''
                
                if ($gain == 4 | $gain == 5) {
                   matrix `xf' = (1, `phit')
                   forvalues h = 1/`hf' {
                      matrix `pf' = `c''*`xf''
                      matrix `phit' = `phit' + `pf'   
                      matrix `xf' = (1, `pf')
                   }
                   matrix `phit' = `phit'/(`hf'+1)    
                }       
                else if $gain == 7 {                
                   matrix `xf' = (1, `phit', 0)
                   forvalues h = 1/`hf' {
                      matrix `pf' = `c''*`xf''
                      matrix `phit' = `phit' + `pf'
                      matrix `xf' = (1, `pf', 0)
                   }
                   matrix `phit' = `phit'/(`hf'+1)
                } 
                else if $gain == 6 { 
                   matrix `phit' = `phit' + 3*`c'[1,1]
                }   
                else if $gain == 8 {
                   matrix `xf' = (1, `phit', `xcurr'[1,2])
                   forvalues h = 1/`hf' {
                      matrix `pf' = `c''*`xf''
                      matrix `phit' = `phit' + `pf'
                      matrix `xf' = (1, `pf', `xf'[1,2])
                   }
                   matrix `phit' = `phit'/(`hf'+1) 
                }
                else if ($gain == 9 ) {
                   matrix `xf' = (1, 0)
                   forvalues h = 1/`hf' {
                      matrix `pf' = `c''*`xf''
                      matrix `phit' = `phit' + `pf'
                      matrix `xf' = (1, 0)
                   }
                   matrix `phit' = `phit'/(`hf'+1)
                }
                //if $gain == 10 {
                //   scalar `ac' = (-1+sqrt(1+4*`c'[2,1]))/2
                //   matrix `phit' = (`phit'-`c'[3,1]*`xcurr'[1,2])/(1-`c'[3,1])
                //   matrix `xf' = (1, `phit')
                //   forvalues h = 1/`hf' {
                //      matrix `pf' = (`c'[1,1]/(1-`c'[3,1]), `ac'^4)*`xf''
                //      matrix `phit' = `phit' + `pf'   
                //      matrix `xf' = (1, `pf')
                //   }
                //   matrix `phit' = (`phit'/(`hf'+1))*(1-`c'[3,1]) + `c'[3,1]*`xcurr'[1,3] 
                //}   
               
             }
                 
          if ($gain < 6 | $gain > 6) {
             matrix inflfullexp[`yri',1] = `phit'
          }    
          else {
             matrix inflfullexp[`yri',1] = `phit'+`infl'
          }
             
          if ($gain == 4 | $gain == 5 | $gain == 6 | $gain == 7 | $gain == 10)  {
             matrix acfull[`yri',1] = `c'[2,1] 
             if ($gain == 7 | $gain == 10) {
                matrix mafull[`yri',1] = `c'[3,1] 
             }
          } 
      }
   }
       
end 






   
capture program drop learnalg
program learnalg
   version 10.1
   syntax anything   
   tempname phit pf xf gcoh infl linfl llinfl ao lao dinfl dlinfl ageqtr ///
            c cc x xcurr y R smean smeanx perr1 perr2 perr3 perr4 k infl4 linfl4 l4infl ac winfl4 wlinfl4 th winfl wlinfl
   scalar `k' = `anything'
   matrix inflexp = J((2009-1871)*4,113,-999)
   matrix ac = J((2009-1871)*4,113,-999)
   matrix ma = J((2009-1871)*4,113,-999) 
   matrix c = J((2009-1871)*4,113,-999) 
   matrix smean = J((2009-1871)*4,113,-999) 
   matrix smeanx = J((2009-1871)*4,113,-999) 
   
   // specifications: gain == 
   // 1: simple mean with weight as in Depression babies
   // 2: simple mean with Kalman filter gain
   // 3: simple mean with decreasing gain
   // 4: AR(1) (weighted least squares) with decreasing gain
   // 5: AR(1) with stochastic gradient learning and decreasing gain
   // 6: IMA(1,1) with decreasing gain
   // 7: ARMA(1,1) with decreasing gain
   // 8: MA(1) with decreasing gain
   
   local yrst = $yrst
   forvalues cohj = `yrst'/1984  {   
       forvalues yr = `yrst'/2009 {
          forvalues q = 1/4 {
             local yri = (`yr'-1872)*4+`q'      // yrinfl starts in qtr. 1872:01           
             scalar `infl' = $pred[`yri',1]          
             scalar `ageqtr' = (`yr'-`cohj'-$stage)*4+`q'-1   // ass. born in first qtr.
             if `ageqtr' > 2*`k'  {
               if $gain == 1 {       
                   scalar `gcoh' = 1/(1+(1/`gcoh')*((`ageqtr'-1)/`ageqtr')^`k')           // as in Depression babies
               }
               else if $gain == 2 {
                   scalar `gcoh' = 1/(1+(1/(`gcoh'+`k')))                                 // Kalman filter
               }
               else {
                   scalar `gcoh' = `k'/(`ageqtr')                                         // Marcet-Sargent 
               }        
             }
             else {
               scalar `gcoh' = 0.5      
             }
             
             
             //initialize learning process (these initial conditions don't matter because gain = 1 for long time after)
             
             if (`yr' < `yrst'+20) {     //& `q' == 1
             
                if $gain < 4 {                       // simple weighted average                       
                   matrix `R' = 1
                   //matrix `x1' = 1   
                   matrix `phit' = `infl'
                   matrix `cc' = 0
                   matrix `x' = 1
                   //matrix `x3' = 1
                   //matrix `x2' = 1 
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                }
                
                else if $gain == 4 {                          // Recursive least squares: AR(1)
                   //matrix `R' = (1, `infl'\ `infl', `infl'^2)  // dispersion in fitted exp looks identical with this one
                   matrix `R' = (1, 0\ 0, 0)
                   //matrix `x1' = (1,`infl')   
                   matrix `phit' = `infl'
                   matrix `cc' = (`phit' \ 0)     // dispersion in fitted exp looks identical with 0, 0.99 as starting value
                   matrix `x' = (1,0)
                   //matrix `x3' = (1,0)
                   //matrix `x2' = (1,0)      
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                }
                
                else if $gain == 5 {                          // Stochastic gradient: AR(1)
                   matrix `R' = 1
                   //matrix `x1' = (1,`infl')   
                   matrix `phit' = `infl'
                   matrix `cc' = (`phit' \ 0)
                   matrix `x' = (1,0)
                   //matrix `x3' = (1,0)
                   //matrix `x2' = (1,0) 
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                }
                
                else if $gain == 6 {                          // IMA(1,1) 
                   matrix `R' = (1, 0\ 0, 1)   // do not impose zero intercept in est., only in forecasting
                   matrix `phit' = `infl'
                   matrix `cc' =  (0 \ 0)
                   matrix `x' = (0, 0)
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                else if $gain == 7 {                          // Recursive least squares: ARMA(1,1) 
                   matrix `R' = (1, 0, 0 \ 0, 1, 0 \ 0, 0, 1)
                   matrix `phit' = `infl'
                   matrix `cc' = (0 \ 1 \ 0)     
                   matrix `x' = (1,0, 0)
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                else if $gain == 8 {                          // AR(2)
                   matrix `R' = (1, 0, 0 \ 0, 1, 0 \ 0, 0, 1)
                   matrix `phit' = `infl'
                   matrix `cc' = (0 \ 1 \ 0)     
                   matrix `x' = (1, 0, 0)
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                else if $gain == 9 {                          // MA(1)
                   matrix `R' = (1, 0\ 0, 0)
                   matrix `phit' = `infl'
                   matrix `cc' = (`phit' \ 0)     // dispersion in fitted exp looks identical with 0, 0.99 as starting value
                   matrix `x' = (1,0)
                   scalar `smean' = `infl'
                   scalar `smeanx' = 0
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                else if $gain == 10 { 
                   matrix `R' = (1, 0 \ 0, 1)
                   matrix `phit' = `infl'
                   matrix `cc' = (0.01 \ 0.5 )     
                   matrix `x' = (0 \ 0 )
                   scalar `smean' = 0.01
                   scalar `smeanx' = 0.01
                   matrix `perr1' = 0
                   matrix `perr2' = 0
                   matrix `perr3' = 0
                   matrix `perr4' = 0
                }
                
                matrix `c' = `cc'
                
                   
             }
             
             
             //updating of parameter estimates 
             
             else if `yr' < `cohj'+76 {
             
                if $qtr == 0 {
                   scalar `linfl' = $pred[`yri'-4,1]
                   scalar `llinfl' = $pred[`yri'-8,1]
                   scalar `dlinfl' = `linfl'-$pred[`yri'-8,1]
                   scalar `dinfl' = `infl'-$pred[`yri'-4,1]
                }
                else if $qtr == 1 {
                   scalar `linfl' = $pred[`yri'-1,1]
                   scalar `llinfl' = $pred[`yri'-2,1]
                   scalar `l4infl' = $pred[`yri'-4,1]
                   scalar `dlinfl' = `linfl'-$pred[`yri'-2,1]
                   scalar `dinfl' = `infl'-$pred[`yri'-1,1]
                   scalar `lao' = ($pred[`yri'-1,1]+$pred[`yri'-2,1]+$pred[`yri'-3,1]+$pred[`yri'-4,1])/4
                   scalar `ao' = ($pred[`yri',1]+$pred[`yri'-1,1]+$pred[`yri'-2,1]+$pred[`yri'-3,1])/4
                   scalar `wlinfl' = (4*$pred[`yri'-4,1]+3*$pred[`yri'-3,1]+2*$pred[`yri'-2,1]+$pred[`yri'-1,1])/10
                   scalar `winfl' = (4*$pred[`yri',1]+3*$pred[`yri'-1,1]+2*$pred[`yri'-2,1]+$pred[`yri'-3,1])/10                   
                   scalar `infl4' = ($pred[`yri',1]+$pred[`yri'-1,1]+$pred[`yri'-2,1]+$pred[`yri'-3,1])/4
                   scalar `linfl4' = ($pred[`yri'-4,1]+$pred[`yri'-5,1]+$pred[`yri'-6,1]+$pred[`yri'-7,1])/4
                }
                
                if $gain == 3 {
                   matrix `x' = 1
                }
                else if ($gain == 4 | $gain == 5) {
                   matrix `x' = (1,`linfl') 
                }
                else if $gain == 6 {
                   if $qtr == 0 { 
                      matrix `x' = (1,`perr4')
                   }   
                   else {
                      matrix `x' = (1,`perr1')
                   }   
                }
                else if $gain == 7 { 
                   if $qtr == 0 { 
                      matrix `x' = (1, `linfl', `perr4')
                   }   
                   else {
                      matrix `x' = (1, `linfl', `perr1'+`perr2'+`perr3'+`perr4')
                      //matrix `x' = (1, -c[3,1]*`llinfl'+`linfl', -c[3,1]*`perr2'+`perr1')
                   }   
                }
                else if $gain == 8 { 
                   if $qtr == 0 { 
                      matrix `x' = (1, `linfl', `llinfl')   
                   }   
                   else {
                      matrix `x' = (1, `linfl', `llinfl')
                   }
                }
                else if $gain == 9 { 
                   if $qtr == 0 { 
                      matrix `x' = (1, `perr4')
                   }   
                   else {
                      matrix `x' = (1, `perr1')
                   }   
                }
                else if $gain == 10 { 
                   if $qtr == 0 { 
                      matrix `x' = (1, `lao')
                   }   
                   else {
                      matrix `x' = (1, `linfl')
                   }
                }
                
                if $gain == 10 {
                   scalar `y' = `infl'
                }
                else if $gain ~= 6 {
                   scalar `y' = `infl'
                }
                else {
                   scalar `y' = `dinfl'
                }
                
                if $gain > 5 {
                   matrix `perr4' = `perr3'
                   matrix `perr3' = `perr2'
                   matrix `perr2' = `perr1'
                }
                
               
                //if ($gain ~= 6 ) {
                   matrix `perr1' = `y'-`cc''*`x''
                //}   
                //else {
                //   matrix `perr1' = `y'-`cc'[2,1]*`x'[1,2] 
                //}
                //else { 
                //   matrix `perr1' = `infl'-`cc''*(1 \ `linfl' \ `perr1')   // this presumes RW interpretation
                   // matrix `perr1' = `dinfl'-`cc'[2,1]*`x'[1,3] 
                //}  
                   
                
                //recursive updating step
                //if $gain ~= 10 {
                  if $gain ~= 5 {
                     matrix `R' = `R' + `gcoh'*(`x''*`x'-`R')      
                  }
                  matrix `cc' = `cc' + `gcoh'*invsym(`R')*`x''*`perr1'     
                //}
                //else {
                //   matrix `R' = `R' + `gcoh'*(`x''*`x'-`R')     
                //   matrix `cc' = `cc' + `gcoh'*invsym(`R')*`x''*(`y'-`cc''*`x'')
                //}
                
                //parameter constraints
                matrix `c' = `cc'
                if ($gain == 6) {
                   matrix `c'[2,1] = max(`c'[2,1],-0.999)
                   matrix `c'[2,1] = min(`c'[2,1],0)
                }
                if $gain == 7 {
                   matrix `c'[2,1] = `c'[2,1]-max(`cc'[2,1]-1,0)   // important to do manipulations on `c', not `cc'
                   matrix `c'[2,1] = `c'[2,1]-min(`cc'[2,1],0)
                   matrix `c'[1,1] = `c'[1,1] + max(`cc'[2,1]-1,0)*`smeanx' + min(`cc'[2,1],0)*`smeanx'
                   matrix `c'[3,1] = max(`c'[3,1],-0.999)
                   matrix `c'[3,1] = min(`c'[3,1],0.999)
                }
                          
                scalar `smean' = `smean' + `gcoh'*(`infl'-`smean') 
                scalar `smeanx' = `smeanx' + `gcoh'*(`linfl'-`smeanx')  
                    
                if $gain == 3 {
                   matrix `xcurr' = 1 
                }
                else if ($gain == 4 | $gain == 5) {
                   matrix `xcurr' = (1,`infl') 
                }
                else if $gain == 6 {
                  matrix `xcurr' = (1,`perr1')
                }
                else if $gain == 7 {
                  matrix `xcurr' = (1, `infl', `perr1'+`perr2'+`perr3'+`perr4')
                }
                else if $gain == 8 {
                  matrix `xcurr' = (1, `infl', `linfl')
                }
                else if $gain == 9 {
                  matrix `xcurr' = (1, `perr1')
                }
                else if $gain == 10 {
                  matrix `xcurr' = (1, `infl')
                }
                 
                if ($qtr == 1 )  {
                   local hf = 4*$hor-1
                }
                else {
                   local hf = $hor-1
                }
                  
                matrix `phit' = `c''*`xcurr''
                
                if ($gain == 4 | $gain == 5) {
                   matrix `xf' = (1, `phit')
                   forvalues h = 1/`hf' {
                      matrix `pf' = `c''*`xf''
                      matrix `phit' = `phit' + `pf'   
                      matrix `xf' = (1, `pf')
                   }
                   matrix `phit' = `phit'/(`hf'+1)    
                }       
                else if $gain == 7 {                
                 //  matrix `xf' = (1, `phit', 0)
                 //  forvalues h = 1/`hf' {
                 //     matrix `pf' = `c''*`xf''
                 //     matrix `phit' = `phit' + `pf'
                 //     matrix `xf' = (1, `pf', 0)
                 //  }
                 //  matrix `phit' = `phit'/(`hf'+1)
                } 
                else if $gain == 6 { 
                   matrix `phit' = `phit' + 3*`c'[1,1]
                }   
                else if $gain == 8 {
                   matrix `xf' = (1, `phit', `xcurr'[1,2])
                   forvalues h = 1/`hf' {
                      matrix `pf' = `c''*`xf''
                      matrix `phit' = `phit' + `pf'
                      matrix `xf' = (1, `pf', `xf'[1,2])
                   }
                   matrix `phit' = `phit'/(`hf'+1) 
                }
                else if ($gain == 9 ) {
                   matrix `xf' = (1, 0)
                   forvalues h = 1/`hf' {
                      matrix `pf' = `c''*`xf''
                      matrix `phit' = `phit' + `pf'
                      matrix `xf' = (1, 0)
                   }
                   matrix `phit' = `phit'/(`hf'+1)
                }
                //if $gain == 10 {
                //   scalar `ac' = (-1+sqrt(1+4*`c'[2,1]))/2
                //   matrix `phit' = (`phit'-`c'[3,1]*`xcurr'[1,2])/(1-`c'[3,1])
                //   matrix `xf' = (1, `phit')
                //   forvalues h = 1/`hf' {
                //      matrix `pf' = (`c'[1,1]/(1-`c'[3,1]), `ac'^4)*`xf''
                //      matrix `phit' = `phit' + `pf'   
                //      matrix `xf' = (1, `pf')
                //   }
                //   matrix `phit' = (`phit'/(`hf'+1))*(1-`c'[3,1]) + `c'[3,1]*`xcurr'[1,3] 
                //}   
                //else if $gain == 10 {                
                //   matrix `xf' = (1, `phit')
                //   forvalues h = 1/`hf' {
                //      matrix `pf' = (`c'[1,1], `c'[2,1]+0.5)*`xf''
                //      matrix `phit' = `phit' + `pf'
                //      matrix `xf' = (1, `pf')
                //   }
                //   matrix `phit' = `phit'/(`hf'+1)
                //} 
               
             }
             
             if $gain == 10 {
                matrix inflexp[`yri',`cohj'-1871] = `phit'  //`smean' + acfull[`yri',1]*(`infl'-`smean') 
             } 
             else if $gain ~= 6  {
                matrix inflexp[`yri',`cohj'-1871] = `phit'
             }    
             else {
                matrix inflexp[`yri',`cohj'-1871] = `phit' + `infl'
             }
             
             
             matrix c[`yri',`cohj'-1871] = `c'[1,1]
             if ($gain > 3 ) {    
                 matrix ac[`yri',`cohj'-1871] = `c'[2,1] 
             }    
             else {
                 matrix ac[`yri',`cohj'-1871] = 0
             }    
             matrix smean[`yri',`cohj'-1871] = `smean' 
             matrix smeanx[`yri',`cohj'-1871] = `smeanx' 
             if ($gain ==7 | $gain == 8 | $gain == 10)  {
                matrix ma[`yri',`cohj'-1871] = `c'[3,1] 
             }
             else {
                matrix ma[`yri',`cohj'-1871] = 0
             }   
              
             
          } 
       }
    }
end 



capture program drop nllearn    
program nllearn                //defines nonlinear function, initializes parameters, and calc function to be fit by command nl prop
  version 10.1  
  syntax [varlist(default=none)] if, at(name)
  tokenize `varlist'                // varlist includes depvar, but not the intercept, and not the learning-from-experience terms
  local n : word count `varlist' 
  if $ar > 3 {
     local n = `n'-1                // no intercept in these cases
  }
  tempname a b k c 
  local atn = colsof(`at')         // includes expl var | learning-from-experience prelim coeff | intercept
  local pn = `atn'-`n'             // # of learning-from-experience parameters: b, c, k
  if `pn' == 3 {
     scalar `a' = `at'[1,`n'+2]    
     scalar `b' = `at'[1,`n']
     scalar `k' = `at'[1,`n'+3]
     scalar `c' = `at'[1,`n'+1]
  }
  else if (`pn' == 2 & $ar < 4) {
     scalar `a' = `at'[1,`n'+1]
     scalar `b' = `at'[1,`n']
     scalar `k' = `at'[1,`n'+2]
     scalar `c' = 0
  }
  else if (`pn' == 2 & $ar > 3) {
     scalar `a' = 0
     scalar `b' = `at'[1,1]
     scalar `k' = `at'[1,2]
     scalar `c' = 0
  }
  else if `pn' == 1 {
     scalar `a' = 0
     scalar `b' = 1
     scalar `k' = `at'[1,1]
     scalar `c' = 0
  }
  
  qui replace `1' = `a' `if'      // token `1' is the depvar, here start replacing it with fitted value
  forvalues i=2/`n' {
       tempname ipar
       scalar `ipar' = `at'[1,`i'-1]                  // retrieve parameter values
       qui replace `1' = `1' + `ipar'*``i'' `if'      // calculates function value 
  }  
  if `k' ~= chkk {
     if $ar ~= 1 {
        learnalg `k'
     }
     else {
        global pred = "inflgas"
        learnalg `k'
        matrix gasinflexp = inflexp
        global pred = "yrinfl"
        learnalg `k'    
     }   
     scalar chkk = `k'       
  }
  tempvar iexploc1 iexploc2 xnl mxnl
  qui gen `iexploc1' = (yyyy-1872)*4+q-1 `if'       // uses data up to previous qtr
  qui gen `iexploc2' = yyyy-age-1871 `if'
  qui gen `xnl' = inflexp[`iexploc1',`iexploc2'] `if' 
  bysort yyyyq: egen `mxnl' = mean(`xnl') `if'
  if `pn' == 3 {
     if $ar == 1 { 
        replace `1' = `1' + `b'*`xnl' + `c'*gasinflexp[`iexploc1',`iexploc2'] `if'  
     }
     else if $ar == 3 {
        replace `1' = `1' + `b'*smean[`iexploc1',`iexploc2'] + `c'*(`xnl'-smean[`iexploc1',`iexploc2']) `if'  //
     }
  }
  else {
     if $ar == 5 {    
        replace `1' = `1' + l1spf + `b'*(`xnl' - l1spf)  `if'   
     } 
     else if $ar == 6 {    
        replace `1' = `1' + `mxnl' + `b'*(`xnl' - `mxnl')  `if'   
     }
     else {
        replace `1' = `1' + `b'*`xnl'  `if'
     }   
  }
end 


capture program drop nlgrid
program nlgrid, eclass
   version 10
   syntax varlist [if] [, KLOW(real 1) KSTEP(real 1) KHI(real 1) AR(int 1)] 
   marksample touse
   gettoken depvar explvar: varlist 
   tempname rss b v minrss binit binitmod vinit kmin chk Vc Vt V eb
   if `ar' == 1 {
      local varnl = 3    // number of coefficients in prelim regression that are b, c, or intercept a
      local parnl "b c a k"
   }
   else if `ar' == 2 {
      local varnl = 2
      local parnl "b a k"
   }
   else if `ar' == 3 {
      local varnl = 3
      local parnl "b c a k"
   }
   else if `ar' == 4 {
      local varnl = 1    // even though no beta estimated here, set to 1 to get nvarnoc = 0 below  
      local parnl = "k"
   }
   else if `ar' == 5 {
      local varnl = 1
      local parnl = "b k"
   }
   else if `ar' == 6 {
      local varnl = 1
      local parnl = "b k"
   }
   
   forvalues k = `klow'(`kstep')`khi' {
      learnalg `k' 
      tempvar iexploc1 iexploc2 xnl ynl mxnl smean arcomp ressq xnldiff dvdiff
      tempname tempmat
      qui gen `iexploc1' = (yyyy-1872)*4+q-1 if `touse'        //  uses data up to previous qtr
      qui gen `iexploc2' = yyyy-age-1871 if `touse'
      qui gen `xnl' = inflexp[`iexploc1',`iexploc2'] if `touse'  
      bysort yyyyq: egen `mxnl' = mean(`xnl') if `touse'
      qui gen `smean' = smean[`iexploc1',`iexploc2'] if `touse'
      qui gen `arcomp' = `xnl'-`smean' if `touse'
	  
      if `ar' == 1 {
         global pred = "inflgas"
         matrix `tempmat' = inflexp
         learnalg `k'
         qui gen `ynl' = inflexp[`iexploc1',`iexploc2']
         qui reg `varlist' `xnl' `ynl' if `touse'
         scalar `rss' = e(rss)
         matrix `b' = e(b)
         matrix `v' = vecdiag(e(V))
         drop `ynl' 
         global pred = "yrinfl"
         matrix inflexp = `tempmat'
      }
      else if `ar' == 2 {                       // main specification
         qui reg `varlist' `xnl' if `touse'    
         scalar `rss' = e(rss)
         matrix `b' = e(b)
         matrix `v' = vecdiag(e(V))
      }
      else if `ar' == 3 {                       // breaks mean component from AR component
         qui reg `varlist' `smean' `arcomp' if `touse'    
         scalar `rss' = e(rss)
         matrix `b' = e(b)
         matrix `v' = vecdiag(e(V))
      }
      else if `ar' == 4 {                      // restricted beta = 1
         qui gen `ressq' = (`depvar' - `xnl')^2 if `touse'
         qui summarize `ressq'
         scalar `rss' = r(sum)
         matrix `b' = (1)
         matrix `v' = (1)
         drop `ressq'
      }
      else if `ar' == 5 {                      // restricted f = spf
	     qui gen `dvdiff' = `depvar' - l1spf if `touse'
         qui gen `xnldiff' = `xnl' - l1spf if `touse'
         qui reg `dvdiff' `xnldiff' if `touse', noconstant
         scalar `rss' = e(rss)
         matrix `b' = e(b)
         matrix `v' = vecdiag(e(V))
         drop `xnldiff' `dvdiff'
      }
      else if `ar' == 6 {                      // restricted f = mean(LfE forecast)
         qui gen `dvdiff' = `depvar' - `mxnl' if `touse'
         qui gen `xnldiff' = `xnl' - `mxnl' if `touse'
         qui reg `dvdiff' `xnldiff' if `touse', noconstant
         scalar `rss' = e(rss)
         matrix `b' = e(b)
         matrix `v' = vecdiag(e(V))
         drop `xnldiff' `mxnl' `dvdiff'
      }
      
      if `k' == `klow' {
         scalar `minrss' = `rss'
         matrix `binit' = `b'
         matrix `vinit' = `v'
         scalar `kmin' = `k'
      }
      else if `rss' < `minrss' {
         scalar `minrss' = `rss'
         matrix `binit' = `b'
         matrix `vinit' = `v'
         scalar `kmin' = `k'
      }
      
      drop `xnl' `smean'  `arcomp' `iexploc1' `iexploc2' 
      di as text "k " as result `k'
      di as text "rss " as result `rss'
   }
   di as text "k " as result `kmin'
   di as text "rss " as result `minrss'
   
   local nvar = colsof(`binit')
   local nvarnoc = `nvar'-`varnl'
   local pvar : colfullnames(`binit')      // extract variable names 
   tokenize `pvar'                         // this way variables that reg above dropped are also dropped
   local explvar ""
   local chk1 = 0
   if `ar' ~= 4 {
      qui replace `touse' = e(sample)
   }
   else {
      qui replace `touse' = 0 if `depvar' == .
   }
   
   global ar = `ar'       
   if `nvarnoc' > 0 {
      forvalues h = 1/`nvarnoc' {
         scalar `chk' = `binit'[1,`h']/`vinit'[1,`h']
         if `chk' ~= . {
             if `chk1' == 0 {                       
                matrix `binitmod' = `binit'[1,`h']
             }
             else {
                matrix `binitmod' = `binitmod',`binit'[1,`h'] 
             }
             local explvar "`explvar' ``h''"
             local chk1 = 1
         }
         local q = `h'
      }   
      matrix `binit' = `binitmod',`binit'[1,`q'+1..`q'+`varnl'],`kmin' 
      nl learn @ `depvar'  `explvar' if `touse', vce(cluster cohort) variables(`explvar') parameters(`explvar' `parnl') initial(`binit')  // delta(4e-4)
      if $clustse == 1 {
         matrix `Vc' = e(V)
         matrix `eb' = e(b)
         qui nl learn @ `depvar'  `explvar' if `touse', vce(cluster yyyyq) variables(`explvar') parameters(`explvar' `parnl') initial(`eb')  // delta(4e-4)
         matrix `Vt' = e(V)
         qui nl learn @ `depvar'  `explvar' if `touse', vce(robust) variables(`explvar') parameters(`explvar' `parnl') initial(`eb')  // delta(4e-4)
         matrix `V' = e(V)
         matrix `V' = `Vc' + `Vt' - `V'
         ereturn repost V = `V'
      }   
   }
   else if `ar' ~= 4 { 
      matrix `binit' = `binit'[1, 1..`varnl'],`kmin' 
      nl learn @ `depvar'  if `touse', vce(cluster cohort) parameters(`parnl') initial(`binit') 
      if $clustse == 1 {
         matrix `Vc' = e(V)
         matrix `eb' = e(b)
         qui nl learn @ `depvar'  if `touse', vce(cluster yyyyq) parameters(`parnl') initial(`eb') 
         matrix `Vt' = e(V)
         qui nl learn @ `depvar'  if `touse', vce(robust) parameters(`parnl') initial(`eb') 
         matrix `V' = e(V)
         matrix `V' = `Vc' + `Vt' - `V'
         ereturn repost V = `V'
      }    
   }
   else {     
      matrix `binit' = `kmin' 
      nl learn @ `depvar'  if `touse', vce(cluster cohort) parameters(`parnl') initial(`binit')
      if $clustse == 1 {
         matrix `Vc' = e(V)
         matrix `eb' = e(b)
         qui nl learn @ `depvar'  if `touse', vce(cluster yyyyq) parameters(`parnl') initial(`eb')
         matrix `Vt' = e(V)
         qui nl learn @ `depvar'  if `touse', vce(robust) parameters(`parnl') initial(`eb')
         matrix `V' = e(V)
         matrix `V' = `Vc' + `Vt' - `V'
         ereturn repost V = `V'
      }   
   }
   
   // re-run learn alg so that inflexp available at estimated k
   learnalg  _b[k:_cons]
   scalar chkk = _b[k:_cons]
     
end 



// constant gain estimation for time series


capture program drop nlcglearn    
program nlcglearn                //defines nonlinear function, initializes parameters, and calc function to be fit by command nl prop
  version 10.1  
  syntax [varlist(default=none)] if, at(name)
  
  tempname k 
  scalar `k' = `at'[1,1]
  if `k' ~= chkk {
     constgain `k'  
     scalar chkk = `k'       
  }
  
  tempvar iexploc1 iexploc2 
  qui gen `iexploc1' = (yyyy-1872)*4+q-1 `if'       // uses data up to previous qtr
  replace `1' = inflfullexp[`iexploc1',1]  `if'
 
end 


capture program drop nlcg
program nlcg, eclass
   version 10
   syntax varlist [if] [, KLOW(real 1) KSTEP(real 1) KHI(real 1)] 
   marksample touse
   tempname rss b v minrss binit binitmod vinit kmin chk Vc Vt V eb
   
   local varnl = 1    // even though no beta estimated here, set to 1 to get nvarnoc = 0 below  
   local parnl = "k"
  
   forvalues k = `klow'(`kstep')`khi' {
      constgain `k' 
      tempvar iexploc1 xnl ressq
      qui gen `iexploc1' = (yyyy-1872)*4+q-1 if `touse'        //  uses data up to previous qtr
      qui gen `xnl' = inflfullexp[`iexploc1',1] if `touse'    
      qui gen `ressq' = (`varlist' - `xnl')^2 if `touse'
      qui summarize `ressq'
      scalar `rss' = r(sum)
      matrix `b' = (1)
      matrix `v' = (1)
      drop `ressq'
      
      if `k' == `klow' {
         scalar `minrss' = `rss'
         matrix `binit' = `b'
         matrix `vinit' = `v'
         scalar `kmin' = `k'
      }
      else if `rss' < `minrss' {
         scalar `minrss' = `rss'
         matrix `binit' = `b'
         matrix `vinit' = `v'
         scalar `kmin' = `k'
      }
      
      drop `xnl' `iexploc1' 
      di as text "k " as result `k'
      di as text "rss " as result `rss'
   }
   di as text "k " as result `kmin'
   di as text "rss " as result `minrss'
   
   qui replace `touse' = 0 if `varlist' == .
   matrix `binit' = `kmin' 
   nl cglearn @ `varlist' if `touse', parameters(`parnl') initial(`binit')
     
   // re-run learn alg so that inflexp available at estimated k
   constgain  _b[k:_cons]
   scalar chkk = _b[k:_cons]
   
end 



/******* program for fitted value plots *****************/

capture program drop fitplot    
program fitplot               
  version 8.2  
  syntax , file1(string) file2(string) file3(string)  
  save tempfile, replace
  tempvar fit fityoung fitmid fitold actyoung actmid actold dfitold dfitmid dfityoung dactyoung dactmid dactold stryyyyq yqdate    
  local dep `e(depvar)'          
  predict `fit' if e(sample)
  qui gen `fityoung' = .
  replace `fityoung' = `fit'  if age < 40
  qui gen `fitmid' = .
  replace `fitmid' = `fit'  if (age > 39 & age < 61)
  qui gen `fitold' = .
  replace `fitold' = `fit' if age > 60
  qui gen `actyoung' = .
  replace `actyoung' = `dep' if age < 40
  qui gen `actmid' = .
  replace `actmid' = `dep' if (age > 39 & age < 61)
  qui gen `actold' = .
  replace `actold' = `dep' if age > 60

  collapse (mean) `fityoung' `fitmid' `fitold' `actyoung' `actmid' `actold' mean`dep' `fit', by (yyyy q)
  keep if `actyoung' ~= .
  gen `stryyyyq' = string(yyyy)+"q"+string(q)
  gen `yqdate' = quarterly(`stryyyyq', "yq")
  format `yqdate' %tq
  tsset `yqdate'

  label variable `actyoung' "Age < 40"
  label variable `actmid' "Age 40 to 60"
  label variable `actold' "Age > 60"
  
  label variable `fityoung' "Age < 40 fitted"
  label variable `fitmid' "Age 40 to 60 fitted"
  label variable `fitold' "Age > 60 fitted"
  
  qui gen `dactyoung' = `actyoung' - mean`dep'
  label variable `dactyoung' "Age < 40"
  qui gen `dactmid' = `actmid' - mean`dep'
  label variable `dactmid' "Age 40 to 60"
  qui gen `dactold' = `actold' - mean`dep'
  label variable `dactold' "Age > 60"
  qui gen `dfityoung' = `fityoung' - `fit'
  label variable `dfityoung' "Age < 40 fitted"
  qui gen `dfitmid' = `fitmid' - `fit'
  label variable `dfitmid' "Age 40 to 60 fitted"
  qui gen `dfitold' = `fitold' - `fit'
  label variable `dfitold' "Age > 60 fitted"
   
  tssmooth ma `fityoung'=`fityoung', window(3 1 0) replace
  tssmooth ma `fitmid'=`fitmid', window(3 1 0) replace
  tssmooth ma `fitold'=`fitold', window(3 1 0) replace
  tssmooth ma `actyoung'=`actyoung', window(3 1 0) replace
  tssmooth ma `actmid'=`actmid', window(3 1 0) replace
  tssmooth ma `actold'=`actold', window(3 1 0) replace
  tssmooth ma `dfityoung'=`dfityoung', window(3 1 0) replace
  tssmooth ma `dfitmid'=`dfitmid', window(3 1 0) replace
  tssmooth ma `dfitold'=`dfitold', window(3 1 0) replace
  tssmooth ma `dactyoung'=`dactyoung', window(3 1 0) replace
  tssmooth ma `dactmid'=`dactmid', window(3 1 0) replace
  tssmooth ma `dactold'=`dactold', window(3 1 0) replace
  
 // ms(th) mc(red) mlwidth(thin) recast(connected) 
 
  tsline `fityoung', clcolor(black) clpattern(solid) clwidth(medthick) || ///
      tsline `fitmid', clcolor(red) clpattern(shortdash) clwidth(medthick) || ///
      tsline `fitold', clcolor(blue) clpattern(longdash) clwidth(medthick) || ///
      tsline `actyoung', clcolor(black) clpattern(solid) clwidth(medthick) ms(dh) mc(black) mlwidth(thin) recast(scatter) || ///
      tsline `actmid', clcolor(red) clpattern(solid) clwidth(medthick) ms(x) mc(red) mlwidth(thin)  recast(scatter) || ///
      tsline `actold', clcolor(blue) clpattern(solid) clwidth(medthick) ms(o) mc(blue) mlwidth(thin) recast(scatter) ||, ///
      xtitle("Quarter") ytitle("Expected Inflation")  ///
      legend(label(1 "Age < 40 fit") label(2 "Age 40 to 60") label(3 "Age > 60 fit") label(4 "Age < 40 act.") label(5 "Age 40 to 60 act.") label(6 "Age > 60 act.") cols(3))  ///
      graphregion(color(white))
  graph export "`file1'.eps", replace

  tsline `dfityoung', clcolor(black) clpattern(solid) clwidth(medthick) || ///
      tsline `dfitmid', clcolor(red) clpattern(shortdash) clwidth(medthick) || ///
      tsline `dfitold', clcolor(blue) clpattern(longdash) clwidth(medthick) || ///
      tsline `dactyoung', clcolor(black) clpattern(solid) clwidth(medthick) ms(dh) mc(black) mlwidth(thin) recast(scatter) || ///
      tsline `dactmid', clcolor(red) clpattern(solid) clwidth(medthick) ms(x) mc(red) mlwidth(thin) recast(scatter) || ///
      tsline `dactold', clcolor(blue) clpattern(solid) clwidth(medthick) ms(o) mc(blue) mlwidth(thin) recast(scatter) ||, ///
      xtitle("Quarter") ytitle("Expected Inflation") ///
       legend(label(1 "Age < 40 fit") label(2 "Age 40 to 60 fit") label(3 "Age > 60 fit") label(4 "Age < 40 act.") label(5 "Age 40 to 60 act.") label(6 "Age > 60 act.") cols(3))  ///
      graphregion(color(white))
  graph export "`file2'.eps", replace
  
  tsline `fityoung' if yyyy > 2000, clcolor(black) clpattern(solid) clwidth(medthick) || ///
      tsline `fitmid' if yyyy > 2000, clcolor(red) clpattern(shortdash) clwidth(medthick) || ///
      tsline `fitold' if yyyy > 2000, clcolor(dash) clpattern(longdash) clwidth(medthick) || ///
      tsline `actyoung' if yyyy > 2000, clcolor(black) clpattern(solid) clwidth(medthick) ms(dh) mc(black) mlwidth(thin) recast(scatter) || ///
      tsline `actmid' if yyyy > 2000, clcolor(red) clpattern(solid) clwidth(medthick) ms(x) mc(red) mlwidth(thin) recast(scatter) || ///
      tsline `actold' if yyyy > 2000, clcolor(blue) clpattern(solid) clwidth(medthick) ms(o) mc(blue) mlwidth(thin) recast(scatter) ||, ///
      xtitle("Quarter") ytitle("Expected Inflation") ///
      legend(label(1 "Age < 40 fit") label(2 "Age 40 to 60 fit") label(3 "Age > 60 fit") label(4 "Age < 40 act.") label(5 "Age 40 to 60 act.") label(6 "Age > 60 act.") cols(3))  ///
      graphregion(color(white))
  graph export "`file3'.eps", replace
   
  //activate if for sub-period plots  
  gen dfityoung = `dfityoung'
  gen dfitmid = `dfitmid'
  gen dfitold = `dfitold'
  gen dactyoung = `dactyoung'
  gen dactmid = `dactmid'
  gen dactold = `dactold'
  gen yqdate = `yqdate'
  save temp1, replace 	
  
  use tempfile, clear
end
 
 

capture program drop ipolmedian
program define ipolmedian
  version 8.2
  syntax varlist [, by(varlist)] 
  tempvar med m ntot cf cl ch f l u medval loval hival
  sort `by'
  by `by': egen `m' = mean(`varlist')
  by `by': egen `med' = median(`varlist')
  by `by': egen `ntot' = count(`varlist')   // count only uses non-missing values
  quietly gen `medval' = `varlist' if `varlist' == `med'
  quietly gen `loval' = `varlist' if `varlist' < `med'
  quietly gen `hival' = `varlist' if `varlist' > `med'
  by `by': egen `cl' = count(`loval')
  by `by': egen `ch' = count(`hival')
  by `by': egen `cf' = count(`medval')
  by `by': egen `l' = max(`loval')
  by `by': egen `u' = min(`hival')
  local medname "`varlist'_med" 
  quietly gen `medname' = .
  //quietly replace `medname' = `med' + 0.5*(`ch'/`cf')*(`u'-`med') + 0.5*(`cl'/`cf')*(`l'-`med') if (`cf' > 0 & `ntot' > 1)   // collapses to simple formula for uniform grid
  //quietly replace `medname' = `l' + ((`ntot'/2-`cl')/`cf')*(`u'-`l') if (`cf' > 0 & `ntot' > 1)   // collapses to simple formula for uniform grid
  quietly replace `medname' = `med' + max((`ntot'/2-`cl'-`cf'/2)/(0.5*`cf'),0)*0.5*(`u'-`med') + min((`ntot'/2-`cl'-`cf'/2)/(0.5*`cf'),0)*0.5*(`med'-`l')   if (`cf' > 0 & `ntot' > 1)
  quietly replace `medname' = `med' if (`cf' == 0 | `ch' < 2 | `cl' < 2 | `ntot' == 1 ) 
end


capture program drop wtipolmedian
program define wtipolmedian
  version 8.2
  syntax varlist [, by(varlist)] 
  tempvar med m ntot cf cl ch f l u medval wtx wts lowts hiwts medwts runsum grmed loval hival
  sort `by' `varlist'
  qui gen `wtx' = wt*`varlist' 
  qui gen `wts' = wt if `wtx' ~= . 
  by `by': egen `ntot' = total(`wts')
  by `by': egen `m' = total(`wtx')
  qui replace `m' = `m'/`ntot'
  
  by `by': gen `runsum' = sum(`wts')
  qui replace `runsum' = `runsum'/`ntot'
  qui gen `grmed' = 999
  qui replace `grmed' = `varlist' if `runsum' >= 0.5
  by `by': egen `med' = min(`grmed') 
  
  quietly gen `medwts' = `wts' if `varlist' == `med'
  quietly gen `lowts' = `wts' if `varlist' < `med'
  quietly gen `hiwts' = `wts' if (`varlist' > `med' & `varlist' < .)
  by `by': egen `cl' = total(`lowts') 
  by `by': egen `ch' = total(`hiwts') 
  by `by': egen `cf' = total(`medwts') 
  quietly gen `loval' = `varlist' if `varlist' < `med'
  quietly gen `hival' = `varlist' if (`varlist' > `med' & `varlist' < .)
  by `by': egen `l' = max(`loval') 
  by `by': egen `u' = min(`hival') 
  //qui replace chk1 = `cl' 
  //qui replace chk2 = `ch'
  //qui replace chk3 = `cf' 
  //qui replace chk4 = `med'
  //qui replace chk5 = `ntot' 
 
  local medname "`varlist'_med" 
  quietly gen `medname' = .
  //quietly replace `medname' = `l' + ((`ntot'/2-`cl')/`cf')*(`u'-`l') if (`cf' > 0 & `ntot' > 1)   // collapses to simple formula for uniform grid
  quietly replace `medname' = `med' + max((`ntot'/2-`cl'-`cf'/2)/(0.5*`cf'),0)*0.5*(`u'-`med') + min((`ntot'/2-`cl'-`cf'/2)/(0.5*`cf'),0)*0.5*(`med'-`l')   if (`cf' > 0 & `ntot' > 1)
  quietly replace `medname' = `med' if (`cf' == 0 | `ch' < 2 | `cl' < 2 | `ntot' == 1 )  // cf = 0 can happen when Stata sets median with tie break to ~.5
end


// simple regression with two-way clustered s.e. 

capture program drop clust2reg
program clust2reg, eclass
   version 10
   syntax varlist [if] [, noconstant] 
   marksample touse
   gettoken depvar explvar: varlist 
   tempname Vc Vt V eV eb
   reg `depvar' `explvar' if `touse', `constant' vce(cluster cohort) 
   matrix `Vc' = e(V)
   matrix `eb' = e(b)
   qui reg `depvar' `explvar' if `touse', `constant' vce(cluster yyyyq) 
   matrix `Vt' = e(V)
   qui reg `depvar' `explvar' if `touse', `constant' vce(robust) 
   matrix `V' = e(V)
   matrix `eV' = `Vc' + `Vt' - `V'
   
   scalar e_N=e(N)
   scalar e_df_m = e(df_m)
   scalar e_df_r = e(df_r)
   scalar e_F = e(F)
   scalar e_r2 = e(r2)
   scalar e_rmse = e(rmse)
   scalar e_mss = e(mss)
   scalar e_rss = e(rss)
   scalar e_r2_a = e(r2_a)
   scalar e_ll = e(ll)
   scalar e_ll_0 = e(ll_0)
   
   ereturn post `eb' `eV'
   //ereturn clear
   ereturn scalar N = e_N
   //ereturn scalar df_m = e_df_m
   //ereturn scalar df_r = e_df_r
   //ereturn scalar F= e_F
   //ereturn scalar r2= e_r2
   ereturn scalar rmse = e_rmse
   //ereturn scalar mss = e_mss
   //ereturn scalar rss = e_rss
   ereturn scalar r2_a = e_r2_a
   //ereturn scalar ll = e_ll
   //ereturn scalar ll_0 = e_ll_0
   
end   
    

	
	

// --------------------------------------------------------------------------- //
//																			   //
// Form cohorts and impute missing percentage expectations                     //
//   from categorical responses                                                //
//                                                                             //
// --------------------------------------------------------------------------- //


// UNCOMMENT THIS SECTION TO COMPUTE COHORT LEVEL DATA FROM MICHIGAN SURVEY 
// MICRO DATA

/*


/******************* load michigan survey data ****************************************/


insheet using "michsurvey_stata.txt", clear


******Inflation Percentage******
gen infl1pct = px1q2/100
gen infl1cat = .
replace infl1cat = 0 if px1q1 == 5
replace infl1cat = 1 if px1q1 == 3
replace infl1cat = 2 if (px1q1 == 1 | px1q1 == 2)

gen infl5pct = px5q2/100
gen infl5cat = .
replace infl5cat = 0 if px5q1 == 5
replace infl5cat = 1 if px5q1 == 3
replace infl5cat = 2 if (px5q1 == 1 | px5q1 == 2)

gen infl1cat_0 = 0
replace infl1cat_0 =1 if infl1cat == 0 
replace infl1cat_0 =. if infl1cat == .
gen infl1cat_1 = 0
replace infl1cat_1 =1 if infl1cat == 1 
replace infl1cat_1 =. if infl1cat == .
gen infl1cat_2 = 0
replace infl1cat_2 =1 if infl1cat == 2 
replace infl1cat_2 =. if infl1cat == .

gen infl5cat_0 = 0
replace infl5cat_0 =1 if infl5cat == 0 
replace infl5cat_0 =. if infl5cat == .
gen infl5cat_1 = 0
replace infl5cat_1 =1 if infl5cat == 1 
replace infl5cat_1 =. if infl5cat == .
gen infl5cat_2 = 0
replace infl5cat_2 =1 if infl5cat == 2 
replace infl5cat_2 =. if infl5cat == .


********************* sample requirements *****************************
destring age, replace
keep if (age > 24 & age < 75)
keep if yyyy < 2010


gen mm = yyyymm-yyyy*100
gen q = 1
replace q = 2 if (mm > 3 & mm < 7)   
replace q = 3 if (mm > 6 & mm < 10)
replace q = 4 if mm > 9
gen yyyyq = yyyy*10+q
gen cohort = yyyy - age
replace wt = 1 if wt == .   // no missing values in time periods in which sample weights are available

save basefile, replace



// Note that finfl and finfl5 defined starting from the beginning of the month 
// following the survey month to 12 or 60 months out. 
// By forming cohort data by quarter, we are averaging these monthly 
// numbers within a quarter

	
//overall median
wtipolmedian infl1pct, by(yyyymm)
wtipolmedian infl5pct, by(yyyymm) 
gen fullmedinfl1pct = infl1pct_med
gen fullmedinfl5pct = infl5pct_med
drop infl1pct_med 
drop infl5pct_med


collapse (mean) fullmedinfl1pct fullmedinfl5pct infl1pct infl5pct finfl5 finfl ///
             infl1cat_0 infl5cat_0 infl1cat_1 infl5cat_1 infl1cat_2 infl5cat_2 /// 
            (count) n5 = infl5pct n1 = infl1pct [aweight=wt], by(yyyymm yyyyq yyyy q age)


gen cohort = yyyy-age

xi: clust2reg infl5pct i.yyyymm infl5cat_0 infl5cat_2 if (infl5pct~=. & yyyy > 1981)  
xi: reg infl5pct i.yyyymm infl5cat_0 infl5cat_2  if (infl5pct~=. & yyyy > 1981)   
predict infl5pctimp

xi: clust2reg infl1pct i.yyyymm infl1cat_0 infl1cat_2  if (infl1pct~=.) // & yyyy > 1981)
xi: reg infl1pct i.yyyymm infl1cat_0 infl1cat_2  if (infl1pct~=.) // & yyyy > 1981) 
predict infl1pctimp


collapse (mean) yyyyq cohort fullmedinfl1pct fullmedinfl5pct infl1pct infl5pct finfl5 finfl ///
             infl1cat_0 infl5cat_0 infl1cat_1 infl5cat_1 infl1cat_2 infl5cat_2 infl1pctimp infl5pctimp /// 
            (sum) n5 = n5 n1 = n1, by(yyyy q age)
 
egen chkinfl5 = mean(infl5pct), by(yyyy q)    // only impute when entire year missing. Sometimes 1 to 3 age groups missing. Don't impute those. 
egen chkinfl1 = mean(infl1pct), by(yyyy q)    

gen imp5flag = 0
replace imp5flag = 1 if (chkinfl5 == . & infl5pctimp ~= .)
replace infl5pct = infl5pctimp if chkinfl5 ==.

gen imp1flag = 0
replace imp1flag = 1 if (chkinfl1 == . & infl1pctimp ~= .)
replace infl1pct = infl1pctimp if chkinfl1  ==.

gen err5 = infl5pct-finfl5
gen err1 = infl1pct-finfl

ipolmedian infl1pctimp, by(yyyy q)
ipolmedian infl5pctimp, by(yyyy q) 
gen medinfl1pctimp = infl1pctimp_med
gen medinfl5pctimp = infl5pctimp_med
drop infl1pctimp_med 
drop infl5pctimp_med

ipolmedian infl1pct, by(yyyy q)
ipolmedian infl5pct, by(yyyy q) 
gen medinfl1pct = infl1pct_med
gen medinfl5pct = infl5pct_med
drop infl1pct_med 
drop infl5pct_med

bysort yyyy q: egen meaninfl1pctimp = mean(infl1pctimp)
bysort yyyy q: egen meaninfl5pctimp = mean(infl5pctimp)

bysort yyyy q: egen meaninfl1pct = mean(infl1pct)
bysort yyyy q: egen meaninfl5pct = mean(infl5pct)


save cohfile, replace

 
 */
 

// --------------------------------------------------------------------------- //
//																			   //
// Summary figures										                       //
//   							                                               //
// --------------------------------------------------------------------------- //


use macrofile, clear
gen stryyyyq = string(yyyy)+"q"+string(q)
gen yqdate = quarterly(stryyyyq, "yq")
format yqdate %tq
tsset yqdate
tssmooth ma mainfl = yrinfl, window(3 1) 
keep if yyyy > 1876
tsline mainfl, clcolor(blue) clpattern(solid) clwidth(medthick) ///
        xtitle("Quarter") ytitle("Inflation") ///
             graphregion(color(white))
graph export "`outdir'Infl_hist.eps", replace

keep if yyyy > 1950
tsline yrinfl, clcolor(blue) clpattern(solid) clwidth(medthick) ///
        xtitle("Quarter") ytitle("Inflation") ///
             graphregion(color(white))
graph export "`outdir'Infl_hist_short.eps", replace


use cohfile, clear

gen infl5cat_2_young = .
replace infl5cat_2_young = infl5cat_2 if age < 40
gen infl5cat_2_old = .
replace infl5cat_2_old = infl5cat_2 if age > 60

gen infl5cat_1_young = .
replace infl5cat_1_young = infl5cat_1 if age < 40
gen infl5cat_1_old = .
replace infl5cat_1_old = infl5cat_1 if age > 60

gen infl5cat_0_young = .
replace infl5cat_0_young = infl5cat_0 if age < 40
gen infl5cat_0_old = .
replace infl5cat_0_old = infl5cat_0 if age > 60

gen infl5pct_young = .
replace infl5pct_young = infl5pct if age < 40
gen infl5pct_old = .
replace infl5pct_old = infl5pct if age > 60

gen infl5pctimp_young = .
replace infl5pctimp_young = infl5pctimp if age < 40
gen infl5pctimp_old = .
replace infl5pctimp_old = infl5pctimp if age > 60

gen err5_young = .
replace err5_young = err5 if age < 40
gen err5_old = .
replace err5_old = err5 if age > 60

gen sqerr5_young = .
replace sqerr5_young = err5^2 if age < 40
gen sqerr5_old = .
replace sqerr5_old = err5^2 if age > 60


gen infl1cat_2_young = .
replace infl1cat_2_young = infl1cat_2 if age < 40
gen infl1cat_2_old = .
replace infl1cat_2_old = infl1cat_2 if age > 60

gen infl1cat_1_young = .
replace infl1cat_1_young = infl1cat_1 if age < 40
gen infl1cat_1_old = .
replace infl1cat_1_old = infl1cat_1 if age > 60

gen infl1cat_0_young = .
replace infl1cat_0_young = infl1cat_0 if age < 40
gen infl1cat_0_old = .
replace infl1cat_0_old = infl1cat_0 if age > 60

gen infl1pct_young = .
replace infl1pct_young = infl1pct if age < 40
gen infl1pct_old = .
replace infl1pct_old = infl1pct if age > 60
gen infl1pct_mid = .
replace infl1pct_mid = infl1pct if (age < 61 & age > 39)

gen infl1pctimp_young = .
replace infl1pctimp_young = infl1pctimp if age < 40
gen infl1pctimp_old = .
replace infl1pctimp_old = infl1pctimp if age > 60
gen infl1pctimp_mid = .
replace infl1pctimp_mid = infl1pctimp if (age < 61 & age > 39)

gen err1_young = .
replace err1_young = err1 if age < 40
gen err1_old = .
replace err1_old = err1 if age > 60

gen sqerr1_young = .
replace sqerr1_young = err1^2 if age < 40
gen sqerr1_old = .
replace sqerr1_old = err1^2 if age > 60

qui gen spfloc = (yyyy-1872)*4+q 
qui gen spf = spf[spfloc,1]/100
qui gen spf10 = spf10[spfloc,1]/100



collapse (mean) infl1pct_mid infl1pctimp_mid infl5pct_young infl5pct_old err5_young sqerr5_young err5_old sqerr5_old ///
                err1_young sqerr1_young err1_old sqerr1_old finfl5 imp5flag meaninfl5pct infl5pctimp_old ///
                infl5cat_2_young infl5cat_2_old infl5cat_1_young infl5cat_1_old ///
                infl5cat_0_young infl5cat_0_old infl5pctimp_young meaninfl5pctimp ///
                infl1pct_young infl1pct_old finfl imp1flag meaninfl1pct infl1pctimp_old ///
                infl1cat_2_young infl1cat_2_old infl1cat_1_young infl1cat_1_old ///
                infl1cat_0_young infl1cat_0_old infl1pctimp_young meaninfl1pctimp spf infl1pct infl5pct ///
                fullmedinfl1pct fullmedinfl5pct, by (yyyy q)

gen rmse1_young = sqrt(sqerr1_young)
gen rmse5_young = sqrt(sqerr5_young)
gen rmse1_old = sqrt(sqerr1_old)
gen rmse5_old = sqrt(sqerr5_old)

gen dinfl1pct_young = infl1pct_young-meaninfl1pct
gen dinfl1pctimp_young = infl1pctimp_young-meaninfl1pctimp
gen dinfl1pct_old = infl1pct_old-meaninfl1pct
gen dinfl1pctimp_old = infl1pctimp_old-meaninfl1pctimp
gen dinfl1pct_mid = infl1pct_mid-meaninfl1pct
gen dinfl1pctimp_mid = infl1pctimp_mid-meaninfl1pctimp

gen dinfl5pct_young = infl5pct_young-meaninfl5pct
gen dinfl5pctimp_young = infl5pctimp_young-meaninfl5pctimp
gen dinfl5pct_old = infl5pct_old-meaninfl5pct
gen dinfl5pctimp_old = infl5pctimp_old-meaninfl5pctimp

gen stryyyyq = string(yyyy)+"q"+string(q)
gen yqdate = quarterly(stryyyyq, "yq")
format yqdate %tq
tsset yqdate

//construct 4-qtr moving average of cross-sectional deviations for young and old
tssmooth ma dinfl5pct_young=dinfl5pct_young, window(3 1 0) replace
tssmooth ma dinfl5pct_old=dinfl5pct_old, window(3 1 0) replace
tssmooth ma dinfl1pct_young=dinfl1pct_young, window(3 1 0) replace
tssmooth ma dinfl1pct_old=dinfl1pct_old, window(3 1 0) replace
tssmooth ma dinfl1pct_mid=dinfl1pct_mid, window(3 1 0) replace
tssmooth ma dinfl1pctimp_young=dinfl1pctimp_young, window(3 1 0) replace
tssmooth ma dinfl1pctimp_old=dinfl1pctimp_old, window(3 1 0) replace
tssmooth ma dinfl1pctimp_mid=dinfl1pctimp_mid, window(3 1 0) replace


twoway tsline dinfl1pct_young, clcolor(black) clpattern(solid) clwidth(medthick) ms(dh) mc(black) mlwidth(thin) recast(scatter) || ///
       tsline dinfl1pct_mid, clcolor(red) clpattern(solid) clwidth(medthick) ms(x) mc(red) mlwidth(thin) recast(scatter) || ///
       tsline dinfl1pct_old, clcolor(blue) clpattern(solid) clwidth(medthick) ms(o) mc(blue) mlwidth(thin) recast(scatter) ||, ///
       xtitle("Quarter") ytitle("Expected Inflation") ylabel(-.02(.01).02) ///
       legend(label(1 "Age < 40") label(2 "Age 40 to 60") label(3 "Age > 60") cols(3))  ///
       graphregion(color(white))
graph export "`outdir'Inflexp1_ages_intro.eps", replace



     
gen impshade1 = 0.02 if imp1flag == 1
gen impshade2 = 0.02 if (imp1flag == 0 & infl1pct_young ~=.)
twoway bar impshade1 yqdate, bcolor(gs10) base(-0.02) || ///
       bar impshade2 yqdate, bcolor(gs14) base(-0.02) || ///
       tsline dinfl1pct_young, clcolor(black) clpattern(solid) clwidth(medthick) ms(dh) mc(black) mlwidth(thin) recast(scatter) || ///
       tsline dinfl1pct_mid, clcolor(red) clpattern(solid) clwidth(medthick) ms(x) mc(red) mlwidth(thin) recast(scatter) || ///
       tsline dinfl1pct_old, clcolor(blue) clpattern(solid) clwidth(medthick) ms(o) mc(blue) mlwidth(thin) recast(scatter) ||, ///
       xtitle("Quarter") ytitle("Expected Inflation") ylabel(-.02(.01).02) ///
       legend(label(3 "Age < 40") label(4 "Age 40 to 60") label(5 "Age > 60") label(1 "Imputed") label(2 "Actual") order(3 4 5 1 2) cols(3))  ///
       graphregion(color(white))
graph export "`outdir'Inflexp1_ages.eps", replace
drop impshade1 impshade2 


tsline dinfl1pct_young, clcolor(black) clpattern(solid) clwidth(medthick) ms(dh) mc(black) mlwidth(thin) recast(scatter) || ///
       tsline dinfl1pct_mid, clcolor(red) clpattern(solid) clwidth(medthick) ms(x) mc(red) mlwidth(thin) recast(scatter) || ///
       tsline dinfl1pct_old, clcolor(blue) clpattern(solid) clwidth(medthick) ms(o) mc(blue) mlwidth(thin) recast(scatter)  || ///
       tsline dinfl1pctimp_young, clcolor(black) clpattern(solid) clwidth(medthick) || ///
       tsline dinfl1pctimp_mid, clcolor(red) clpattern(shortdash) clwidth(medthick) || ///
       tsline dinfl1pctimp_old, clcolor(blue) clpattern(longdash) clwidth(medthick) || if tin(1967q1,2009q4), ///
         xtitle("Quarter") ytitle("Expected Inflation")  ///
         legend(label(1 "Age < 40") label(2 "Age 40 to 60") label(3 "Age > 60") label(4 "< 40 imp.")  label(5 "40 to 60 imp.") label(6 "> 60 imp.") cols(3))  ///
         graphregion(color(white))
graph export "`outdir'Inflexp1_impdiff.eps", replace






// ----------------------------------------------------------------------------------------------- //
//                                                                                                 //
// estimation                                                                                      //
//                                                                                                 //
// ----------------------------------------------------------------------------------------------- //


capture log close
log using "`outdir'Stataout.log", text replace


global pred "yrinfl"   // choose macro variable to feed into learning algorithm
global yrst 1872       // starting point for macro history data

use cohfile, clear


qui gen iexploc = (yyyy-1872)*4+q-1  
qui gen l1infl = yrinfl[iexploc,1]
qui gen l2infl = yrinfl[iexploc-4,1]
qui gen lao = (yrinfl[iexploc,1] + yrinfl[iexploc-1,1] + yrinfl[iexploc-2,1] + yrinfl[iexploc-3,1])/4
qui gen laoeld = (inflelderly[iexploc,1] + inflelderly[iexploc-1,1] + inflelderly[iexploc-2,1] + inflelderly[iexploc-3,1])/4
qui gen laogas = (inflgas[iexploc,1] + inflgas[iexploc-1,1] + inflgas[iexploc-2,1] + inflgas[iexploc-3,1])/4
qui gen l1infleldage = age*(laoeld-lao)/100
qui gen l1inflgasage = age*(laogas-lao)/100    // with four-qtr inflation no post-1953 gaps in series
qui gen frage = age/100

qui gen spfloc = (yyyy-1872)*4+q   // For SPF use current quarter forecast
qui gen spf = spf[spfloc,1]/100
qui gen spf10 = spf10[spfloc,1]/100
qui gen l1spf = spf[spfloc-1,1]/100
qui gen l2spf = spf[spfloc-2,1]/100
qui gen l3spf = spf[spfloc-3,1]/100
qui gen spfage = (age-25)*spf
qui gen d1spf = spf-l1spf
qui gen d2spf = l1spf-l2spf
qui gen d3spf = l2spf-l3spf



// Base specifications
//  code calls nlgrid program to run nonlinear least squares estimation 


global stage = 0
global clustse = 1

local h = 1     // Set to 1 for 1-year expectations and 5 for 5-year expectations
global hor = 1 


// Baseline AR(1) specification

global gain = 4  
scalar chkk = 0

eststo r3: xi: nlgrid infl`h'pct i.yyyyq if (infl`h'pct ~=.), klow(2.8) kstep(0.2) khi(4.2) ar(2)
fitplot, file1("`outdir'infl`h'expfit_ar1") file2("`outdir'infl`h'expfit_ar2") file3("`outdir'infl`h'expfit_ar_recent")
esttab r3 using "`outdir'infl`h'reg_0.tex", ar2 scalars(rmse) nostar nonotes b(3) se(3) sfmt(4) label replace

scalar chkk = 0
eststo r4: xi: nlgrid infl`h'pct i.yyyyq if (infl`h'pct ~=. & imp`h'flag == 0), klow(2.8) kstep(0.2) khi(4.2) ar(2) 
fitplot, file1("`outdir'infl`h'expfit_ar1_noimp") file2("`outdir'infl`h'expfit_ar2_noimp") file3("`outdir'infl`h'expfit_ar_recent_noimp") 

esttab r3 r4 using "`outdir'infl`h'reg.tex", ar2 scalars(rmse) nostar nonotes b(3) se(3) sfmt(4) label replace


// AR(1) restricted specifications w/o time dummies -- only makes sense for non-imputed data
// constraining beta with time dummies does not make sense: if learning-from-experience fully explains expectations then no need for time dummies


global gain = 4  
scalar chkk = 0
eststo r41: nlgrid infl`h'pct if (infl`h'pct ~=. & imp`h'flag == 0), klow(4) kstep(0.2) khi(4) ar(5)   // no time dummies, zero intercept, spf


scalar chkk = 0
eststo r411: nlgrid infl`h'pct if (infl`h'pct ~=. & imp`h'flag == 0), klow(4) kstep(0.2) khi(4) ar(6)   // no time dummies, zero intercept, f = mean LfE forecast

esttab r41 r411 using "`outdir'infl`h'reg_11.tex", ar2 scalars(rmse) nostar nonotes b(3) se(3) sfmt(4) label replace



// theta fixed and f = average LfE forecast

local h = 1
global hor = 1
local k = 3.044   // theta estimate from AR(1)    
learnalg `k'
qui gen iexploc1 = (yyyy-1872)*4+q-1     // uses data up to previous qtr: in learnalg program (`yr'-1872)*4+`q' uses infl up to current qtr
qui gen iexploc2 = yyyy-age-1871
qui gen inflave1 = inflexp[iexploc1,iexploc2] 
bysort yyyyq: egen minflave1 = mean(inflave1)
qui gen dinfl1pct = infl1pct-minflave1
qui gen dinflave1 = inflave1-minflave1

eststo r5: clust2reg dinfl1pct dinflave if (infl1pct ~=. & imp1flag == 0), noconstant
esttab r5 using "`outdir'infl`h'reg_5.tex", ar2 scalars(rmse) nostar nonotes b(3) se(3) sfmt(4) label replace



//with CPI-E (for the elderly) 


local h = 1     // Set to 1 for 1-year expectations and 5 for 5-year expectations
global hor = 1 

global gain = 4
scalar chkk = 0
eststo r11: xi: nlgrid infl`h'pct i.yyyyq if (infl`h'pct ~=. & l1infleldage ~=.), klow(2.6) kstep(0.2) khi(3) ar(2) 

scalar chkk = 0
eststo r12: xi: nlgrid infl`h'pct i.yyyyq frage l1infleldage if (infl`h'pct ~=. & l1infleldage ~=.), klow(3.6) kstep(0.2) khi(4) ar(2) 

esttab r11 r12 using "`outdir'infl`h'reg_cpie.tex", ar2 nostar nonotes b(3) se(3) label replace



//with CPI-Gasoline


local h = 1     // Set to 1 for 1-year expectations and 5 for 5-year expectations
global hor = 1 

global gain = 4
scalar chkk = 0

eststo r21: xi: nlgrid infl`h'pct i.yyyyq if (infl`h'pct ~=. & l1inflgasage ~=.), klow(2.8) kstep(0.2) khi(3.2) ar(2) 

scalar chkk = 0
eststo r22: xi: nlgrid infl`h'pct i.yyyyq frage l1inflgasage if (infl`h'pct ~=. & l1inflgasage ~=.), klow(3.6) kstep(0.2) khi(3.8) ar(2) 

esttab r21 r22 using "`outdir'infl`h'reg_cpigas.tex", ar2 nostar nonotes b(3) se(3) label replace

scalar chkk = 0
eststo r222: xi: nlgrid infl`h'pct i.yyyyq frage l1inflgasage if (infl`h'pct ~=. & l1inflgasage ~=. & yyyy > 1972), klow(2.8) kstep(0.2) khi(3.2) ar(2) 
fitplot, file1("`outdir'infl`h'expfit_cpigas_sub1") file2("`outdir'infl`h'expfit_cpigas_sub2") file3("`outdir'infl`h'expfit_cpigas_sub_recent") 

esttab r222 using "`outdir'infl`h'reg_cpigas_sub.tex", ar2 nostar nonotes b(3) se(3) label replace





// ----------------------------------------------------------------------------------------------- //
//                                                                                                 //
// comparison with sticky expectations                                                             //
//                                                                                                 //
// ----------------------------------------------------------------------------------------------- //



use cohfile, clear

qui gen iexploc = (yyyy-1872)*4+q-1  
qui gen l1infl = yrinfl[iexploc,1]
qui gen l2infl = yrinfl[iexploc-4,1]
qui gen lao = (yrinfl[iexploc,1] + yrinfl[iexploc-1,1] + yrinfl[iexploc-2,1] + yrinfl[iexploc-3,1])/4

qui gen spfloc = (yyyy-1872)*4+q   // For SPF use current quarter forecast
qui gen spf = spf[spfloc,1]/100
qui gen spf10 = spf10[spfloc,1]/100
qui gen l1spf = spf[spfloc-1,1]/100
qui gen l2spf = spf[spfloc-2,1]/100
qui gen l3spf = spf[spfloc-3,1]/100
 
global gain = 4
global stage = 0

//expand to full balanced panel
gen mydate = yq(yyyy,q)
tsset age mydate, quarterly
tsfill, full

gen myddate = dofq(mydate)
replace yyyy = year(myddate)
replace q = quarter(myddate)
qui gen iexploc1 = (yyyy-1872)*4+q-1     // uses data up to previous qtr: in learnalg program (`yr'-1872)*4+`q' uses infl up to current qtr
qui gen iexploc2 = yyyy-age-1871
qui gen iexploc3 = yyyy-74-1871 

// put in missing inflation data after tsfill
qui replace l1infl = yrinfl[iexploc1,1]
qui replace l1spf = spf[spfloc-1,1]/100


local h = 1
global hor = 1
local k = 3.044   // from AR(1)    
local cg = 0.0180  
local b = 0.672
local nwlag1 = 5

constgain 0.0324
qui gen optinflfullave1 = inflfullexp[iexploc1,1]
constgain `cg'  
qui gen inflfullave1 = inflfullexp[iexploc1,1]
scalar chkk = 0
learnalg `k'   
qui gen inflavesm1 = smean[iexploc1,iexploc2]
qui gen inflavesmx1 = smeanx[iexploc1,iexploc2]
qui gen inflave1 = inflexp[iexploc1,iexploc2] 
qui gen ac1 = ac[iexploc1,iexploc2]
qui gen ma1 = ma[iexploc1,iexploc2]
qui gen c1 = c[iexploc1,iexploc2]
qui gen fitexp1 = `b'*inflave1
qui gen expadj1 = infl5pct-fitexp1


save tempfile, replace


qui gen a1 = (ac1^3+2*ac1^2+3*ac1+4)*c1/4
qui gen acc1 = (ac1^4+ac1^3+ac1^2+ac1)/4

gen infl1pct_young = .
replace infl1pct_young = infl1pct if age < 40
gen infl1pct_old = .
replace infl1pct_old = infl1pct if age > 60
gen inflave1_young = .
replace inflave1_young = inflave1 if age < 40
gen inflave1_old = .
replace inflave1_old = inflave1 if age > 60
gen a1_young = .
replace a1_young = a1 if age < 40
gen a1_old = .
replace a1_old = a1 if age > 60


collapse (mean) infl1pct_young infl1pct_old inflave1_young inflave1_old lao acc1  ///
                infl1pct err1 expadj1 fitexp1 l1infl l2infl imp1flag meaninfl1pct inflave1 ///
                inflfullave1 optinflfullave1 inflavesm1 inflavesmx1 finfl fullmedinfl1pct spf spf10 l1spf ///
                ac1 ma1 a1 c1 a1_young a1_old yyyy q yyyyq, by(mydate)    

format mydate %tq
tsset mydate, quarterly

qui replace err1 = . if imp1flag ~= 0
qui replace infl1pct = . if imp1flag ~= 0

//sticky information expectations for 5-year horizon require 5-year SPF
// 1-yr SPF starts in Q 197002
// last four summation terms all with ^8 so that it sums to 1 even though finite

qui gen stickyspf1 = 0.25*(l1.spf+(1-0.25)*l2.spf+(1-0.25)^2*l3.spf+(1-0.25)^3*l4.spf+ ///
                         (1-0.25)^4*l5.spf+(1-0.25)^5*l6.spf+(1-0.25)^6*l7.spf+(1-0.25)^7*l8.spf+ ///
                         (1-0.25)^8*l9.spf+(1-0.25)^8*l10.spf+(1-0.25)^8*l11.spf+(1-0.25)^8*l12.spf)

// force means previous obs treated as one time unit before
// not quite correct NW covar matrix then given the gaps in the 5yr expectations data
// but s.e. almost identical to those obtained when replacing missing obs with lags
// but coefficient biased if replacement by lag, so prefer force 

local nwlag1 = 5

eststo a1: newey meaninfl1pct inflave1, lag(`nwlag1') force
predict fitinflave1 if e(sample)
eststo a2: newey meaninfl1pct inflfullave1, lag(`nwlag1') force
predict fitinflfullave1 if e(sample)
eststo a3: newey meaninfl1pct inflave1 if stickyspf1 ~= ., lag(`nwlag1') force
eststo a4: newey meaninfl1pct stickyspf1, lag(`nwlag1') force
predict fitstickyspf1 if e(sample)

eststo a5: newey meaninfl1pct inflave1 stickyspf1, lag(`nwlag1') force

esttab a1 a2 a3 a4 a5 using "`outdir'infl1agg.tex", ar2 nostar nonotes b(3) se(3) label replace



tsline infl1pct, clcolor(black) clpattern(solid) clwidth(medthick) ms(oh) mc(black) mlwidth(thin) recast(scatter)  || ///
      tsline inflave1, clcolor(red) clpattern(solid) clwidth(medthick)  || ///
      tsline inflfullave1, clcolor(blue) clpattern(longdash) clwidth(medthick)  || ///
      tsline stickyspf1, clcolor(green) clpattern(shortdash) clwidth(medthick) ||, ///
      xtitle("Quarter") ytitle("Expected Inflation") ///
      legend(label(1 "Actual") label(2 "Learning from experience") ///
      label(3 "Constant-gain learning") label(4 "Sticky information") cols(2))  ///
      graphregion(color(white))
graph export "`outdir'infl1constgaincomp_raw.eps", replace

tsline infl1pct, clcolor(black) clpattern(solid) clwidth(medthick) ms(oh) mc(black) mlwidth(thin) recast(scatter)  || ///
      tsline fitinflave1, clcolor(red) clpattern(solid) clwidth(medthick)  || ///
      tsline fitinflfullave1, clcolor(blue) clpattern(longdash) clwidth(medthick)  || ///
      tsline fitstickyspf1, clcolor(green) clpattern(shortdash) clwidth(medthick) ||, ///
      xtitle("Quarter") ytitle("Expected Inflation") ///
      legend(label(1 "Actual") label(2 "Learning from experience") ///
      label(3 "Constant-gain learning") label(4 "Sticky information") cols(2))  ///
      graphregion(color(white))
graph export "`outdir'infl1constgaincomp_fit.eps", replace



*get R2: 

eststo a1rsq: reg meaninfl1pct inflave1
eststo a2rsq: reg meaninfl1pct inflfullave1
eststo a3rsq: reg meaninfl1pct inflave1 if stickyspf1 ~= .
eststo a4rsq: reg meaninfl1pct stickyspf1
eststo a5rsq: reg meaninfl1pct inflave1 stickyspf1 

esttab a1rsq a2rsq a3rsq a4rsq a5rsq using "`outdir'infl1aggrsq.tex", ar2 nostar nonotes b(3) se(3) label replace





// ----------------------------------------------------------------------------------------------- //
//                                                                                                 //
// Investment/borrowing decisions from Survey of Consumer Finances                                 //
//                                                                                                 //
// ----------------------------------------------------------------------------------------------- //



capture program drop impprobit
program impprobit
   version 10
   syntax varlist [if] [pweight] [, vce(passthru) keyvar(varname) intvar(varname)] 
   marksample touse
   tempvar touseorig
   qui gen `touseorig' = `touse'
   if "`weight'" == "pweight" {
      local sumwt "aweight"
   }
   else {
      local sumwt ""
   } 
   forvalues j=1/5 {
       replace `touse' = `touseorig'
       replace `touse' = 0 if (imp ~= `j' & year > 1988)     // use only data from one implicate at a time
       probit `varlist' if `touse' [`weight' `exp'], `vce'
       tempname beta`j' betavar`j'
       matrix `beta`j'' = e(b)
       matrix `betavar`j'' = vecdiag(e(V))'
       tempname p10 p90 iterm
       qui sum `keyvar' if e(sample)==1 [`sumwt' `exp'], detail    // calulate average difference between 90th and 10th pctile of exp returns fitted probabilities
       scalar `p10' = r(p10)
       scalar `p90' = r(p90) 
       
       di as text "p10 = " as result `p10'
       di as text "p90 = " as result `p90'
      
       
       if "`intvar'" ~= "" {
          scalar `iterm' = _b[`intvar']
       }
       else {
          scalar `iterm' = 0
       }   
          
       tempvar pctdiff1 dpctdiff1 pctdiff2 dpctdiff2
       tempname mpct1`j' mpctse1`j' mpct2`j' mpctse2`j' 
       
       predictnl `pctdiff1' = normal(xb() + _b[`keyvar']*(`p90'-`keyvar')) - normal(xb() + _b[`keyvar']*(`p10'-`keyvar')) if e(sample) == 1
       predictnl `pctdiff2' = normal(xb() + (_b[`keyvar']+`iterm')*(`p90'-`keyvar')) - normal(xb() + (_b[`keyvar']+`iterm')*(`p10'-`keyvar')) if e(sample) == 1
       
       predictnl `dpctdiff1' = normalden(xb() + _b[`keyvar']*(`p90'-`keyvar'))*`p90' - normalden(xb() + _b[`keyvar']*(`p10'-`keyvar'))*`p10' if e(sample) == 1
       predictnl `dpctdiff2' = normalden(xb() + (_b[`keyvar']+`iterm')*(`p90'-`keyvar'))*`p90' - normalden(xb() + (_b[`keyvar']+`iterm')*(`p10'-`keyvar'))*`p10' if e(sample) == 1
         
       qui sum `pctdiff1' if e(sample)==1 [`sumwt' `exp']
       scalar `mpct1`j'' = r(mean)
       qui sum `dpctdiff1' if e(sample)==1 [`sumwt' `exp']
       scalar `mpctse1`j'' = r(mean)*_se[`keyvar']    // delta-method standard error 
       qui sum `pctdiff2' if e(sample)==1 [`sumwt' `exp']
       scalar `mpct2`j'' = r(mean)
       qui sum `dpctdiff2' if e(sample)==1 [`sumwt' `exp']
       scalar `mpctse2`j'' = r(mean)*_se[`keyvar']    // delta-method standard error 
   }
   tempname beta mpct1 mpctse1 mpct2 mpctse2
   matrix `beta' = (`beta1'+`beta2'+`beta3'+`beta4'+`beta5')/5
   local nbeta = colsof(`beta')
   forvalues i=1/`nbeta'  {
     tempname betaouti
     local betai = `beta'[1,`i']
     local betasei = sqrt((`betavar1'[`i',1]+`betavar2'[`i',1]+`betavar3'[`i',1]+`betavar4'[`i',1]+`betavar5'[`i',1])/5+ ///
           1.2*(1/4)*((`beta1'[1,`i']-`betai')^2+(`beta2'[1,`i']-`betai')^2+(`beta3'[1,`i']-`betai')^2+(`beta4'[1,`i']-`betai')^2+(`beta5'[1,`i']-`betai')^2))
     matrix `betaouti' = (`betai',`betasei')   
     di "average coefficient, s.e. = " 
     matrix list `betaouti'
   }
   scalar `mpct1' = (`mpct11'+`mpct12'+`mpct13'+`mpct14'+`mpct15')/5
   scalar `mpctse1' = sqrt((`mpctse11'^2+`mpctse12'^2+`mpctse13'^2+`mpctse14'^2+`mpctse15'^2)/5+ ///
                      1.2*(1/4)*((`mpct11'-`mpct1')^2+(`mpct12'-`mpct1')^2+(`mpct13'-`mpct1')^2+(`mpct14'-`mpct1')^2+(`mpct15'-`mpct1')^2))
   scalar `mpct2' = (`mpct21'+`mpct22'+`mpct23'+`mpct24'+`mpct25')/5
   scalar `mpctse2' = sqrt((`mpctse21'^2+`mpctse22'^2+`mpctse23'^2+`mpctse24'^2+`mpctse25'^2)/5+ ///
                      1.2*(1/4)*((`mpct21'-`mpct2')^2+(`mpct22'-`mpct2')^2+(`mpct23'-`mpct2')^2+(`mpct24'-`mpct2')^2+(`mpct25'-`mpct2')^2))                   
   di as text "sample average fitted prob diff between 90th and 10th pctile of experienced return (intdum = 0) = " as result `mpct1'
   di as text "sample average fitted prob diff between 90th and 10th pctile s.e. (intdum = 0) = " as result `mpctse1'
   di as text "sample average fitted prob diff between 90th and 10th pctile of experienced return (intdum = 1) = " as result `mpct2'
   di as text "sample average fitted prob diff between 90th and 10th pctile s.e. (intdum = 1) = " as result `mpctse2'
   
end 


capture program drop imptobit
program imptobit
   version 10
   syntax varlist [if] [pweight] [, ll(passthru) vce(passthru) keyvar(varname)] 
   marksample touse
   tempvar touseorig
   qui gen `touseorig' = `touse'
   if "`weight'" == "pweight" {
      local sumwt "aweight"
   }
   else {
      local sumwt ""
   } 
   forvalues j=1/5 {
       replace `touse' = `touseorig'
       replace `touse' = 0 if (imp ~= `j' & year > 1988)     // use only data from one implicate at a time
       tobit `varlist' if `touse' [`weight' `exp'], `ll' `vce'
       tempname beta`j' betavar`j' sigma`j'
       matrix `beta`j'' = e(b)
       matrix `betavar`j'' = vecdiag(e(V))'
       //scalar `sigma`j'' = `beta`j''[1,e(df_m)+2]
       scalar `sigma`j'' = _b[/sigma]
       tempname p10 p90 
       qui sum `keyvar' if e(sample)==1 [`sumwt' `exp'], detail    // calulate average difference between 90th and 10th pctile of exp returns fitted probabilities
       scalar `p10' = r(p10)
       scalar `p90' = r(p90) 
      
       di as text "p10 = " as result `p10'
       di as text "p90 = " as result `p90'
      
       tempvar pctdiff1 dpctdiff1 
       tempname mpct1`j' mpctse1`j'  
       
       // compute E(y|x), i.e., not conditioned on y > 0, Wooldridge p. 523
       
       //qui gen `xbhi' = (xb() + _b[`keyvar']*(`p90'-`keyvar'))
       //qui gen `xblo' = (xb() + _b[`keyvar']*(`p10'-`keyvar'))
       //qui gen `xact' = `xb' + _b[b:_cons]*`xnl'
       //qui gen `imhi' = normalden(`xbhi'/`sig')/normal(`xbhi'/`sig')
       //qui gen `imlo' = normalden(`xblo'/`sig')/normal(`xblo'/`sig')
       //qui gen `Ey' = `xact' + `sig'*normalden(`xact'/`sig')/normal(`xact'/`sig')  ///
       //                if (e(sample) == 1 & `depvar' > 0)   // fitted value for non-cens. obs 
       //
       //qui gen `pctdiff' = `xbhi' + `sig'*`imhi' - `xblo' - `sig'*`imlo' if e(sample)==1 
       //qui gen `dpctdiff' = ///
       //   `p90' + `sig'*(normalden(`xbhi'/`sig')*(-`xbhi'/`sig')*(`p90'/`sig')*normal(`xbhi'/`sig')^(-1) - (normalden(`xbhi'/`sig')^2)*(normal(`xbhi'/`sig')^(-2))*(`p90'/`sig')) ///
       // - `p10' - `sig'*(normalden(`xblo'/`sig')*(-`xblo'/`sig')*(`p10'/`sig')*normal(`xblo'/`sig')^(-1) - (normalden(`xblo'/`sig')^2)*(normal(`xblo'/`sig')^(-2))*(`p10'/`sig')) ///
       //   if e(sample)==1  
         
       
       predictnl `pctdiff1' = normal((xb() + _b[`keyvar']*(`p90'-`keyvar'))/`sigma`j'')*(xb() + _b[`keyvar']*(`p90'-`keyvar')) + `sigma`j''*normalden((xb() + _b[`keyvar']*(`p90'-`keyvar'))/`sigma`j'')  ///
                               - normal((xb() + _b[`keyvar']*(`p10'-`keyvar'))/`sigma`j'')*(xb() + _b[`keyvar']*(`p10'-`keyvar')) - `sigma`j''*normalden((xb() + _b[`keyvar']*(`p10'-`keyvar'))/`sigma`j'') if e(sample)==1
                                
       predictnl `dpctdiff1' = normal((xb() + _b[`keyvar']*(`p90'-`keyvar'))/`sigma`j'')*`p90' - normal((xb() + _b[`keyvar']*(`p10'-`keyvar'))/`sigma`j'')*`p10' if e(sample)==1
       
       qui sum `pctdiff1' if e(sample)==1 [`sumwt' `exp']
       scalar `mpct1`j'' = r(mean)
       qui sum `dpctdiff1' if e(sample)==1 [`sumwt' `exp']
       scalar `mpctse1`j'' = r(mean)*_se[`keyvar']    // delta-method standard error 
   }
   tempname beta mpct1 mpctse1 
   matrix `beta' = (`beta1'+`beta2'+`beta3'+`beta4'+`beta5')/5
   local nbeta = colsof(`beta')
   forvalues i=1/`nbeta'  {
     tempname betaouti
     local betai = `beta'[1,`i']
     local betasei = sqrt((`betavar1'[`i',1]+`betavar2'[`i',1]+`betavar3'[`i',1]+`betavar4'[`i',1]+`betavar5'[`i',1])/5+ ///
           1.2*(1/4)*((`beta1'[1,`i']-`betai')^2+(`beta2'[1,`i']-`betai')^2+(`beta3'[1,`i']-`betai')^2+(`beta4'[1,`i']-`betai')^2+(`beta5'[1,`i']-`betai')^2))
     matrix `betaouti' = (`betai',`betasei')   
     di "average coefficient, s.e. = " 
     matrix list `betaouti'
   }
   scalar `mpct1' = (`mpct11'+`mpct12'+`mpct13'+`mpct14'+`mpct15')/5
   scalar `mpctse1' = sqrt((`mpctse11'^2+`mpctse12'^2+`mpctse13'^2+`mpctse14'^2+`mpctse15'^2)/5+ ///
                      1.2*(1/4)*((`mpct11'-`mpct1')^2+(`mpct12'-`mpct1')^2+(`mpct13'-`mpct1')^2+(`mpct14'-`mpct1')^2+(`mpct15'-`mpct1')^2))               
   di as text "sample average fitted prob diff between 90th and 10th pctile of experienced return = " as result `mpct1'
   di as text "sample average fitted prob diff between 90th and 10th pctile s.e. = " as result `mpctse1'
   
end 



capture program drop estmimp
program estmimp
   version 10
   syntax anything [if] [pweight] [, ll(passthru) vce(passthru)] 
   marksample touse
   tempvar touseorig
   qui gen `touseorig' = `touse'
   forvalues j=1/5 {
       replace `touse' = `touseorig'
       replace `touse' = 0 if (imp ~= `j' & year > 1988)     // use only data from one implicate at a time
       `anything' if `touse' [`weight' `exp'], `ll' `vce'
       tempname beta`j' betavar`j'
       matrix `beta`j'' = e(b)
       matrix `betavar`j'' = vecdiag(e(V))'
       }
   tempname beta 
   matrix `beta' = (`beta1'+`beta2'+`beta3'+`beta4'+`beta5')/5
   local nbeta = colsof(`beta')
   forvalues i=1/`nbeta'  {
     tempname betaouti
     local betai = `beta'[1,`i']
     local betasei = sqrt((`betavar1'[`i',1]+`betavar2'[`i',1]+`betavar3'[`i',1]+`betavar4'[`i',1]+`betavar5'[`i',1])/5+ ///
           1.2*(1/4)*((`beta1'[1,`i']-`betai')^2+(`beta2'[1,`i']-`betai')^2+(`beta3'[1,`i']-`betai')^2+(`beta4'[1,`i']-`betai')^2+(`beta5'[1,`i']-`betai')^2))
     matrix `betaouti' = (`betai',`betasei')   
     di "average coefficient, s.e. = " 
     matrix list `betaouti'
     }
end 



// load basefile.dta based on Malmendier and Nagel (2011) 


use "`dr'/Dropbox/temp/Inflexp/scfbasefile.dta", clear


replace svmort = . if svmort < 0    // one erroneous obs in 1968

gen mort = svmort
replace mort = 0 if (svmort ~= . & varmort == 1)  // i.e., mort  = fixed rate mortgage (first mortgage)
gen vmort = svmort 
replace vmort = 0 if (svmort ~= . & varmort == 0)
replace vmort = . if year < 1980

gen cash = fw-svstk-svbond
gen inc = faminc
gen yyyy = year
gen tass = fw+svhome    // svhome is home value   
gen logtass = log(tass)
gen loginc = log(inc) 

gen dyearmort = yyyy-newmort
replace dyearmort = . if newmort == 0

local k = 3.044   // from AR(1) 
local h = 1     // Set to 1 for 1-year expectations and 5 for 5-year expectations
global hor = 1 
global gain = 4  // AR(1)
scalar chkk = 0
qui gen iexploc1 = (yyyy-1872)*4+1     // uses data up to end of qtr 1 of survey year
qui gen iexploc2 = yyyy-age-1871
global pred "yrinfl"
scalar chkk = 0
local h = 1
global hor = 1 
global gain = 4
learnalg `k'
qui gen inflave = inflexp[iexploc1,iexploc2] 


gen nw = tass-svmort
gen refmort = mort
replace refmort = 0 if (dyearmort > 0 | dyearmort == .) 
gen refvmort = vmort
replace refvmort = 0 if (dyearmort > 0 | dyearmort == .) 

replace heloc = 0 if heloc == . 

save scfhhfile, replace 

collapse (sum) inc tass nw mort svbond vmort refmort refvmort heloc fwnr fw ///
        (count) ninc=inc ntass=tass nnw=nw nmort=mort nsvbond=svbond nvmort=vmort nheloc=heloc ///
                nrefmort=refmort nrefvmort=refvmort nfwnr=fwnr nfw = fw [pweight=relwt], by(yyyy cohort age)

replace nw = nw/nnw
replace tass = tass/ntass
replace inc = inc/ninc
replace svbond = svbond/nsvbond
replace mort = mort/nmort
replace vmort = vmort/nvmort
replace refmort = refmort/nrefmort
replace refvmort = refvmort/nrefvmort
replace heloc = heloc/nheloc
replace fwnr = fwnr/nfwnr
replace fw = fw/nfw

gen logmort = log(mort+1)    // many helocs have variable interest rates, but subtracting helocs here makes almost no difference
gen tmort = log(mort+vmort+1) 
gen logsvbond = log(svbond+1) 
gen loginc = log(inc+1) 
gen lognw = log(nw+1) 
gen logfwnr = log(fwnr+1) 
gen logfw = log(fw+1) 
gen logvmort = log(vmort+1)
gen logrefmort = log(refmort+1)
gen logrefvmort = log(refvmort+1)

//gen debtinc = mort/inc
//gen debtnw = mort/nw
//gen netdebtinc = (mort-svbond)/inc
//gen netdebtnw = (mort-svbond)/nw 

qui gen iexploc1 = (yyyy-1872)*4+1     // uses data up to end of qtr 1 of survey year
qui gen iexploc2 = yyyy-age-1871
gen inflave = inflexp[iexploc1,iexploc2] 
gen linflave = inflexp[iexploc1-1,iexploc2] 

replace logrefvmort = . if yyyy < 1983
replace logrefmort = . if yyyy < 1983
replace refvmort = . if yyyy < 1983
replace refmort = . if yyyy < 1983

tabstat mort heloc vmort refvmort refmort

latabstat logmort logsvbond logrefmort logrefvmort  loginc lognw, stats(mean sd p10 median p90) f(%5.2f)

 
egen minflave = mean(inflave), by(yyyy)
gen dinflave = inflave-minflave
tabstat dinflave inflave if logmort ~=., stats(mean sd)
egen mlogmort = mean(logmort), by(yyyy)  
gen dlogmort = logmort-mlogmort


//xi: reg logmort i.yyyy i.age inflave loginc logfw if logmort ~=., vce(cluster yyyy)  
 

eststo scf1: xi: reg logmort i.yyyy i.age inflave loginc lognw if logmort ~=., vce(cluster yyyy)  

eststo scf2: xi: reg logsvbond i.yyyy i.age inflave loginc lognw if logsvbond ~=., vce(cluster yyyy) 

eststo scf3: xi: reg logmort i.yyyy i.age inflave loginc lognw if (logmort ~=. & yyyy > 1982), vce(cluster yyyy) 

eststo scf4: xi: reg logsvbond i.yyyy i.age inflave loginc lognw if (logsvbond ~=. & yyyy > 1982), vce(cluster yyyy)

eststo scf5: xi: reg logrefmort i.yyyy i.age inflave loginc lognw if (logrefmort ~=. & yyyy > 1982), vce(cluster yyyy) 

eststo scf6: xi: reg logrefvmort i.yyyy i.age inflave loginc lognw if (logrefvmort ~=. & yyyy > 1982), vce(cluster yyyy) 

// Note: using average of inflave and linflave delivers similar results in last two regressions

esttab scf1 scf2 scf3 scf4 scf5 scf6 using "`outdir'infl`h'reg_scf.tex", ar2 scalars(rmse) nostar nonotes b(2) se(2) sfmt(4) label replace

// sub-sample check 

xi: reg logrefmort i.yyyy i.age inflave loginc lognw if (logrefmort ~=. & yyyy > 1982 & yyyy < 2001), vce(cluster yyyy)


//winsorized variables for robustness check 
winsor logmort, gen(Wlogmort) p(0.1) lowonly
winsor logrefmort, gen(Wlogrefmort) p(0.2) lowonly  // p(0.1) is zero
gen Wlogrefvmort = logrefvmort
replace Wlogrefvmort = 5.379989 if logrefvmort < 5.379989  // p(0.5) is zero

eststo wscf1: xi: reg Wlogmort i.yyyy i.age inflave loginc lognw if logmort ~=., vce(cluster yyyy)  
eststo wscf3: xi: reg Wlogmort i.yyyy i.age inflave loginc lognw if (logmort ~=. & yyyy > 1982), vce(cluster yyyy) 
eststo wscf5: xi: reg Wlogrefmort i.yyyy i.age inflave loginc lognw if (logrefmort ~=. & yyyy > 1982), vce(cluster yyyy) 
eststo wscf6: xi: reg Wlogrefvmort i.yyyy i.age inflave loginc lognw if (logrefvmort ~=. & yyyy > 1982), vce(cluster yyyy) 

esttab wscf1 wscf3 wscf5 wscf6 using "`outdir'infl`h'reg_scf_wins.tex", ar2 scalars(rmse) nostar nonotes b(2) se(2) sfmt(4) label replace





// Robustness check: synthetic cohort panel with lagged mortgage balance  

tsset cohort yyyy, yearly 

gen l1mort = L1.logmort 
gen l2mort = L2.logmort 
gen l3mort = L3.logmort 
gen l6mort = L6.logmort 

replace l2mort = . if l1mort ~= . 
replace l3mort = . if l1mort ~= . 
replace l6mort = . if l1mort ~= . 
replace l3mort = . if l2mort ~= . 
replace l6mort = . if l2mort ~= .
replace l6mort = . if l3mort ~= . 

replace l1mort = 0 if l1mort == . 
replace l2mort = 0 if l2mort == . 
replace l3mort = 0 if l3mort == . 
replace l6mort = 0 if l6mort == . 

gen logincold = loginc
replace logincold = 0 if age < 66

xi: reg logmort i.yyyy i.age inflave loginc  lognw if yyyy > 1982, vce(cluster yyyy) 




// Robustness check: Use hh-level data

use scfhhfile, clear 

gen lognw = log(1+max(0,nw))
gen logmort = log(mort+1) 
gen logsvbond = log(svbond+1) 
gen logrefmort = log(refmort+1) 
gen logrefvmort = log(refvmort+1) 
gen ref = 0 
replace ref = 1 if (refmort > 0 | refvmort > 0) 

//3 self emp? 
gen occ = occup
replace occ = 1 if (year < 1989 & (occup == 2 | occup == 3))
replace occ = 2 if (year < 1989 & (occup == 4 | occup == 1))
replace occ = 3 if (year < 1989 & (occup == 8 | occup == 5))
replace occ = 4 if (year < 1989 & occup == 6)
replace occ = 4 if (year > 1988 & occup == 5)  // to merge two post 88 categories 
replace occ = 6 if (year < 1989 & occup == 7) 

replace risk = 0 if riskav < 3   // as in Coulibaly and Li (2009) 

gen loginc2 = loginc^2
gen lognw2 = lognw^2 
gen logtmort = log(mort+vmort+1)
gen logtmort2 = log(mort+vmort+1)^2
local controlsfull "i.yyyy i.agegrp edu1 edu2 i.gender retir spart i.race marit child loginc lognw"
local controlsfull2 "i.yyyy i.agegrp edu1 edu2 i.gender retir spart i.race marit child loginc lognw loginc2 lognw2"
local controlsfull3 "i.agegrp edu1 edu2 i.gender retir spart i.race marit child i.yyyy*loginc i.yyyy|lognw i.yyyy|loginc2 i.yyyy|lognw2"
local controlsfull4 "i.yyyy i.agegrp edu1 edu2 i.gender retir spart i.race marit child loginc lognw loginc2 lognw2 logtmort logtmort2 i.refin_ever"
local controlsfull5 "i.yyyy i.agegrp edu1 edu2 i.gender retir spart i.race marit child i.yyyy*loginc i.yyyy|lognw i.yyyy|loginc2 i.yyyy|lognw2 i.yyyy|logtmort i.yyyy|logtmort2 i.refin_ever"
local controls "i.yyyy i.agegrp edu1 edu2 i.gender retir spart risk i.occ i.race marit child loginc lognw"

capture log close
log using hhlevellog, replace  

save temp, replace 
summarize logmort logsvbond inflave
xi: imptobit logmort `controlsfull' inflave if (mort ~=. & loginc ~= . & lognw ~= .) [pweight = relwt], ll(0) vce(cluster year) keyvar(inflave)
xi: imptobit logsvbond `controlsfull'  inflave if (svbond ~=. & loginc ~= . & lognw ~= .) [pweight = relwt], ll(0)  vce(cluster year) keyvar(inflave)
keep if yyyy > 1982
summarize logrefmort logrefvmort varmort inflave
xi: imptobit logrefmort `controlsfull' inflave if (logrefmort ~=. & ref == 1 & loginc ~= . & lognw ~= . & yyyy > 1982) [pweight = relwt], ll(0) vce(cluster year) keyvar(inflave)
xi: imptobit logrefvmort `controlsfull' inflave if (logrefvmort ~=. & ref == 1  & loginc ~= . & lognw ~= . & yyyy > 1982) [pweight = relwt], ll(0)  vce(cluster year) keyvar(inflave)
xi: impprobit varmort `controlsfull' inflave if (varmort~= . & ref == 1 & loginc ~= . & lognw ~= .) [pweight = relwt], vce(cluster year) keyvar(inflave)


use temp, clear 
xi: imptobit logmort `controlsfull2' inflave if (mort ~=. & loginc ~= . & lognw ~= .) [pweight = relwt], ll(0) vce(cluster year) keyvar(inflave)
xi: imptobit logsvbond `controlsfull2'  inflave if (svbond ~=. & loginc ~= . & lognw ~= .) [pweight = relwt], ll(0)  vce(cluster year) keyvar(inflave)
keep if yyyy > 1982
xi: imptobit logrefmort `controlsfull2' inflave if (logrefmort ~=. & ref == 1 & loginc ~= . & lognw ~= . & yyyy > 1982) [pweight = relwt], ll(0) vce(cluster year) keyvar(inflave)
xi: imptobit logrefvmort `controlsfull2' inflave if (logrefvmort ~=. & ref == 1  & loginc ~= . & lognw ~= . & yyyy > 1982) [pweight = relwt], ll(0)  vce(cluster year) keyvar(inflave)
xi: impprobit varmort `controlsfull2' inflave if (varmort~= . & ref == 1 & loginc ~= . & lognw ~= .) [pweight = relwt], vce(cluster year) keyvar(inflave)

use temp, clear 
xi: imptobit logmort `controlsfull3' inflave if (mort ~=. & loginc ~= . & lognw ~= .) [pweight = relwt], ll(0) vce(cluster year) keyvar(inflave)
xi: imptobit logsvbond `controlsfull3'  inflave if (svbond ~=. & loginc ~= . & lognw ~= .) [pweight = relwt], ll(0)  vce(cluster year) keyvar(inflave)
keep if yyyy > 1982
xi: imptobit logrefmort `controlsfull3' inflave if (logrefmort ~=. & ref == 1 & loginc ~= . & lognw ~= . & yyyy > 1982) [pweight = relwt], ll(0) vce(cluster year) keyvar(inflave)
xi: imptobit logrefvmort `controlsfull3' inflave if (logrefvmort ~=. & ref == 1  & loginc ~= . & lognw ~= . & yyyy > 1982) [pweight = relwt], ll(0)  vce(cluster year) keyvar(inflave)
xi: impprobit varmort `controlsfull3' inflave if (varmort~= . & ref == 1 & loginc ~= . & lognw ~= .) [pweight = relwt], vce(cluster year) keyvar(inflave)


//before 1995 refin_ever is all zeros
xi: impprobit varmort `controlsfull4' inflave if (varmort~= . & ref == 1 & loginc ~= . & lognw ~= . & year > 1994) [pweight = relwt], vce(cluster year) keyvar(inflave)

xi: impprobit varmort `controlsfull5' inflave if (varmort~= . & ref == 1 & loginc ~= . & lognw ~= . & year > 1994) [pweight = relwt], vce(cluster year) keyvar(inflave)


log close






